# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""Some method for interacting with input/output from CP2K.

Important methods defined here
------------------------------

update_cp2k_input (:py:func:`.update_cp2k_input`)
    A method for updating a CP2K input file and creating a new one.

read_cp2k_input (:py:func:`.read_cp2k_input`)
    A method to read a CP2K input file.
"""
import logging
import os
import numpy as np
from pyretis.core.box import box_matrix_to_list, box_vector_angles
logger = logging.getLogger(__name__)  # pylint: disable=invalid-name
logger.addHandler(logging.NullHandler())

np.set_printoptions(legacy='1.21')


class SectionNode:
    """A class representing a section in the CP2K input.

    Attributes
    ----------
    title : string
        The title of the section
    parent : string
        The parent section if this node represents a
        sub-section.
    settings : list of strings
        The setting(s) for this particular node.
    data : string
        A section of settings if the node defines several
        settings.
    children : set of objects like :py:class:`.SectionNode`
        A set with the sub-sections of this section.
    level : integer
        An integer to remember how far down this node is.
        E.g. if the level is 2, this node is a sub-sub-section.
        This is used for printing.
    parents : list of strings or None
        A list representing the path from the node to the top
        section.

    """

    def __init__(self, title, parent, settings, data=None):
        """Initialise a node.

        Parameters
        ----------
        title : string
            The title of the section.
        parent : object like :py:class:`.SectionNode`
            The parent if this section is a sub-section.
        settings : list of strings
            The settings defined in this section.
        data : list of strings, optional
            A section of settings.

        """
        self.title = title
        self.parent = parent
        self.settings = settings
        if data:
            self.data = list(data)
        else:
            self.data = []
        self.children = set()
        self.level = 0
        self.parents = None

    def add_child(self, child):
        """Add a sub-section to the current section."""
        self.children.add(child)

    def get_all_parents(self):
        """Find the path to the top of the tree."""
        parents = [self.title]
        prev = self.parent
        while prev is not None:
            parents.append(prev.title)
            prev = prev.parent
        self.parents = parents[::-1]


def dfs_print(node, visited):
    """Walk through the nodes and print out text.

    Parameters
    ----------
    node : object like :py:class:`.SectionNode`
        The object representing a C2PK section.
    visited : set of objects like :py:class:`.SectionNode`
        The set contains the nodes we have already visited.

    Returns
    -------
    out : list of strings
        These strings represent the CP2K input file.

    """
    out = []
    pre = ' ' * (2 * node.level)
    if not node.settings:
        out.append(f'{pre}&{node.title}')
    else:
        out.append(f'{pre}&{node.title} {" ".join(node.settings)}')
    for lines in node.data:
        out.append(f'{pre}  {lines}')
    visited.add(node)
    for child in node.children:
        if child not in visited:
            for lines in dfs_print(child, visited):
                out.append(lines)
    out.append(f'{pre}&END {node.title}')
    return out


def set_parents(listofnodes):
    """Set parents for all nodes."""
    node_ref = {}

    def dfs_set(node, vis):
        """DFS traverse the nodes."""
        if node.parents is None:
            node.get_all_parents()
            par = '->'.join(node.parents)
            if par in node_ref:
                prev = node_ref.pop(par)
                par1 = f'{par}->{" ".join(prev.settings)}'
                par2 = f'{par}->{" ".join(node.settings)}'
                node_ref[par1] = prev
                node_ref[par2] = node
            else:
                node_ref[par] = node
        vis.add(node)
        for child in node.children:
            if child not in visited:
                dfs_set(child, vis)

    for nodes in listofnodes:
        visited = set()
        dfs_set(nodes, visited)
    return node_ref


def read_cp2k_input(filename):
    """Read a CP2K input file.

    Parameters
    ----------
    filename : string
        The file to open and read.

    Returns
    -------
    nodes : list of objects like :py:class:`.SectionNode`
        The root section nodes found in the file.

    """
    nodes = []
    current_node = None
    with open(filename, 'r', encoding="utf-8") as infile:
        for lines in infile:
            lstrip = lines.strip()
            if not lstrip:
                # skip empty lines
                continue
            if lstrip.startswith('&'):
                strip = lstrip[1:].split()
                if lstrip[1:].lower().startswith('end'):
                    current_node = current_node.parent
                else:
                    if len(strip) > 1:
                        setts = strip[1:]
                    else:
                        setts = []
                    new_node = SectionNode(strip[0].upper(),
                                           current_node, setts)
                    if current_node is None:
                        nodes.append(new_node)
                    else:
                        new_node.level = current_node.level + 1
                        current_node.add_child(new_node)
                    current_node = new_node
            else:
                if current_node is not None:
                    current_node.data.append(lstrip)
    return nodes


def _add_node(target, settings, data, nodes, node_ref):
    """Just add a new node."""
    # check if this is a root node:
    root = target.find('->') == -1
    if root:
        new_node = SectionNode(target, None, settings, data=data)
        nodes.append(new_node)
    else:
        parents = target.split('->')
        title = parents[-1]
        par = '->'.join(parents[:-1])
        if par not in node_ref:
            _add_node(par, None, None, nodes, node_ref)
        parent = node_ref['->'.join(parents[:-1])]
        new_node = SectionNode(title, parent, settings, data=data)
        new_node.level = parent.level + 1
        parent.add_child(new_node)
    node_ref[target] = new_node


def update_node(target, settings, data, node_ref, nodes,
                replace=False):
    """Update the given target node.

    If the node does not exist, it will be created.

    Parameters
    ----------
    target : string
        The target node, on form root->section->subsection
    settings : list of strings
        The settings for the node.
    data : list of strings or dict
        The data for the node.
    node_ref : dict of :py:class:`.SectionNode`
        A dict of all nodes in the tree.
    nodes : list of :py:class:`.SectionNode`
        The root nodes.
    replace : boolean, optional
        If this is True and if the nodes have some data, the already
        existing data will be ignored. We also assume that the data
        is already formatted.

    """
    if target not in node_ref:  # add node
        # TODO: remove decommented try-except construction later
        # try:
        _add_node(target, settings, data, nodes, node_ref)
        # except KeyError:
        #    pass
        return None
    node = node_ref[target]
    new_data = []
    done = set()
    if not replace:
        for line in node.data:
            key = line.split()[0]
            if key in data:
                new_data.append(f'{key} {data[key]}')
                done.add(key)
            else:
                new_data.append(line)
        for key in data:
            if key in done:
                continue
            if data[key] is None:
                new_data.append(str(key))
            else:
                new_data.append(f'{key} {data[key]}')
        node.data = list(new_data)
    else:
        node.data = list(data)
    if settings is not None:
        if replace:
            node.settings = list(settings)
        else:
            node.settings += settings
    return node


def remove_node(target, node_ref, root_nodes):
    """Remove a node (and it's children) from the tree.

    Parameters
    ----------
    target : string
        The target node, on form root->section->subcection.
    node_ref : dict
        A dict with all the nodes.
    root_nodes : list of objects like :py:class:`.SectionNode`
        The root nodes.

    """
    to_del = node_ref.pop(target, None)
    if to_del is None:
        pass
    else:
        # remove all it's children:
        visited = set()
        nodes = [to_del]
        while nodes:
            node = nodes.pop()
            if node not in visited:
                visited.add(node)
                for i in node.children:
                    nodes.append(i)
        # remove the reference to this node from the parent
        parent = to_del.parent
        if parent is None:
            # This is a root node.
            root_nodes.remove(to_del)
        else:
            parent.children.remove(to_del)
        del to_del
        for key in visited:
            _ = node_ref.pop(key, None)


def update_cp2k_input(template, output, update=None, remove=None):
    """Read a template input and create a new CP2K input.

    Parameters
    ----------
    template : string
        The CP2K input file we use as a template.
    output : string
        The CP2K input file we will create.
    update : dict, optional
        The settings we will update.
    remove : list of strings, optional
        The nodes we will remove.

    """
    nodes = read_cp2k_input(template)
    node_ref = set_parents(nodes)
    if update is not None:
        for target in update:
            value = update[target]
            settings = value.get('settings', None)
            replace = value.get('replace', False)
            data = value.get('data', [])
            update_node(target, settings, data, node_ref, nodes,
                        replace=replace)
    if remove is not None:
        for nodei in remove:
            remove_node(nodei, node_ref, nodes)
    with open(output, 'w', encoding='utf-8') as outf:
        for i, nodei in enumerate(nodes):
            vis = set()
            if i > 0:
                outf.write('\n')
            outf.write('\n'.join(dfs_print(nodei, vis)))
            outf.write('\n')


def read_box_data(box_data):
    """Read the box data.

    Parameters
    ----------
    box_data : list of strings
        The settings for the SUBSYS->CELL section.

    Returns
    -------
    out[0] : numpy.array, 1D
        The box vectors, in the correct order.
    out[1] : list of booleans
        The periodic boundary setting for each dimension.

    """
    to_read = {'A': 'vec', 'B': 'vec', 'C': 'vec', 'PERIODIC': 'string',
               'ABC': 'vec', 'ALPHA_BETA_GAMMA': 'vec'}
    data = {}
    for lines in box_data:
        for key, val in to_read.items():
            keyword = f'{key} '
            if lines.startswith(keyword):
                if val == 'vec':
                    data[key] = [float(i) for i in lines.split()[1:]]
                elif val == 'string':
                    data[key] = ' '.join(lines.split()[1:])
    if all(('A' in data, 'B' in data, 'C' in data)):
        box_matrix = np.zeros((3, 3))
        box_matrix[:, 0] = data['A']
        box_matrix[:, 1] = data['B']
        box_matrix[:, 2] = data['C']
        box = box_matrix_to_list(box_matrix)
    elif 'ABC' in data:
        if 'ALPHA_BETA_GAMMA' in data:
            box_matrix = box_vector_angles(
                data['ABC'],
                data['ALPHA_BETA_GAMMA'][0],
                data['ALPHA_BETA_GAMMA'][1],
                data['ALPHA_BETA_GAMMA'][2],
            )
            box = box_matrix_to_list(box_matrix)
        else:
            box = np.array(data['ABC'])
    else:
        box = None
    periodic = []
    periodic_setting = data.get('PERIODIC', 'XYZ')
    for val in ('X', 'Y', 'Z'):
        periodic.append(val in periodic_setting.upper())
    return box, periodic


def read_cp2k_energy(energy_file):
    """Read and return CP2K energies.

    Parameters
    ----------
    energy_file : string
        The input file to read.

    Returns
    -------
    out : dict
        This dict contains the energy terms read from the CP2K energy file.

    """
    data = np.loadtxt(energy_file)
    energy = {}
    for i, key in ((1, 'time'), (2, 'ekin'), (3, 'temp'), (4, 'vpot')):
        try:
            energy[key] = data[:, i]
        except IndexError:
            logger.warning('Could not read energy term %s from CP2kfile %s',
                           key, energy_file)
    if 'ekin' in energy and 'vpot' in energy:
        energy['etot'] = energy['ekin'] + energy['vpot']
    return energy


def read_cp2k_restart(restart_file):
    """Read some info from a CP2K restart file.

    Parameters
    ----------
    restart_file : string
        The file to read.

    Returns
    -------
    pos : numpy.array
        The positions.
    vel : numpy.array
        The velocities.
    box_size : numpy.array
        The box vectors.
    periodic : list of booleans
        For each dimension, the list entry is True if periodic
        boundaries should be applied.

    """
    nodes = read_cp2k_input(restart_file)
    node_ref = set_parents(nodes)
    velocity = 'FORCE_EVAL->SUBSYS->VELOCITY'
    coord = 'FORCE_EVAL->SUBSYS->COORD'
    cell = 'FORCE_EVAL->SUBSYS->CELL'

    atoms, pos, vel = [], [], []

    for posi, veli in zip(node_ref[coord].data, node_ref[velocity].data):
        pos_split = posi.split()
        atoms.append(pos_split[0])
        pos.append([float(i) for i in pos_split[1:4]])
        vel.append([float(i) for i in veli.split()])
    pos = np.array(pos)
    vel = np.array(vel)
    box, periodic = read_box_data(node_ref[cell].data)
    return atoms, pos, vel, box, periodic


def read_cp2k_box(inputfile):
    """Read the box from a CP2K file.

    Parameters
    ----------
    inputfile : string
        The file we will read from.

    Returns
    -------
    out[0] : numpy.array
        The box vectors.
    out[1] : list of booleans
        For each dimension, the list entry is True if periodic
        boundaries should be applied.

    """
    nodes = read_cp2k_input(inputfile)
    node_ref = set_parents(nodes)
    try:
        box, periodic = read_box_data(
            node_ref['FORCE_EVAL->SUBSYS->CELL'].data
        )
    except KeyError:
        logger.warning('No CELL found in CP2K file "%s"', inputfile)
        box = np.array([[100., 0., 0.], [0., 100., 0.], [0., 0., 100.]])
        periodic = [True, True, True]
    return box, periodic


def cp2k_settings(settings, input_path):
    """Read and processes cp2k settings.

    Parameters
    ----------
    settings : dict
        The current input settings..
    input_path : string
        The CP2K input path

    """
    ext = settings['engine'].get('cp2k_format', 'xyz')
    default_files = {'conf': f'initial.{ext}',
                     'template': 'cp2k.inp'}
    settings['engine']['input_files'] = {}
    for key in ('conf', 'template'):
        # Add input path and the input files if input is no given:
        settings['engine']['input_files'][key] = \
            settings['engine'].get(key,
                                   os.path.join(input_path,
                                                default_files[key]))
    nodes = read_cp2k_input(
        settings['engine']['input_files']['template'])
    MD_data = set_parents(nodes)['MOTION->MD'].data

    # Checks temperature
    cp2k_temp_inp = False
    for lines in MD_data:
        if lines.startswith('TEMPERATURE'):
            cp2k_temp_inp = True
            cp2k_temp = float(lines.split()[1])
            if 'temperature' not in settings['system']:
                settings['system']['temperature'] = cp2k_temp
            elif abs(cp2k_temp - settings['system']['temperature']) < 10e-8:
                pass
            else:
                msg = 'Inequal Pyretis and CP2K input temperatures!' +\
                        ' Temperature is set to CP2K input temperature.'
                logger.warning(msg)
                settings['system']['temperature'] = cp2k_temp
            break
    if not cp2k_temp_inp:
        if 'temperature' not in settings['system']:
            msg = 'Temperature not set in CP2K file!' +\
                ' Temperature is set to CP2K default temperature.'
            logger.warning(msg)
        elif abs(300.0 - settings['system']['temperature']) > 10e-8:
            msg = 'Temperature not set in CP2K file!' +\
                ' And temperature in input rst file does not equal 300.0 K!' +\
                ' Temperature is set to CP2K default temperature.'
            logger.warning(msg)
        # CP2K defaults to 300K when temperature is unspecified.
        settings['system']['temperature'] = 300.0



# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""This module checks that the inputs are meaningful.

Main methods defined here
~~~~~~~~~~~~~~~~~~~~~~~~~

check_ensemble (:py:func:`.check_ensemble`)
    Function to check the ensemble settings.

check_interfaces (:py:func:`.check_interfaces`)
    Function to check engine set-up.

check_for_bullshitt (:py:func:`.check_for_bullshitt`)
    Function to compare nested dicts and lists.

check_engine (:py:func:`.check_engine`)
    Function to check engine set-up.

"""
import logging

logger = logging.getLogger(__name__)  # pylint: disable=C0103
logger.addHandler(logging.NullHandler())

__all__ = ['check_interfaces', 'check_for_bullshitt',
           'check_engine', 'check_ensemble']
ENSEMBLE_TASKS = ['retis', 'tis', 'repptis', 'pptis']


def check_ensemble(settings):
    """Check that the ensemble input parameters are complete.

    Parameters
    ----------
    settings : dict
        The settings needed to set up the simulation.

    """
    if 'ensemble' in settings:
        savelambda = []
        for ens in settings['ensemble']:
            if 'interface' in ens:
                savelambda.append(ens['interface'])

                if not ens['interface'] \
                        in settings['simulation']['interfaces']:
                    msg = f"The ensemble interface {ens['interface']} "\
                        "is not present in the simulation interface list"
                    break

            else:
                msg = "An ensemble is present without reference interface"
                break

        if not is_sorted(savelambda):
            msg = "Interface positions in the ensemble simulation "\
                "are NOT properly sorted (ascending order)"

    else:
        msg = "No ensemble in settings"

    if 'msg' in locals():
        raise ValueError(msg)

    return True


def check_engine(settings):
    """Check the engine settings.

    Checks that the input engine settings are correct, and
    automatically determine the 'internal' or 'external'
    engine setting.

    Parameters
    ----------
    settings : dict
        The current input settings.

    """
    msg = []
    if 'engine' not in settings:
        msg += ['The section engine is missing']

    elif settings['engine'].get('type') == 'external':

        if 'input_path' not in settings['engine']:
            msg += ['The section engine requires an input_path entry']

        if 'gmx' in settings['engine'] and \
                'gmx_format' not in settings['engine']:
            msg += ['File format is not specified for the engine']
        elif 'cp2k' in settings['engine'] and \
                'cp2k_format' not in settings['engine']:
            msg += ['File format is not specified for the engine']

    if msg:
        msgtxt = '\n'.join(msg)
        logger.critical(msgtxt)
        return False

    return True


def check_for_bullshitt(settings):
    """Do what is stated.

    Just for the input settings.

    Parameters
    ----------
    settings : dict
        The current input settings.

    """
    if (settings['simulation']['task'] in ENSEMBLE_TASKS and
            len(settings['simulation']['interfaces']) < 3):
        msg = "Insufficient number of interfaces for "\
            f"{settings['simulation']['task']}"

    elif settings['simulation']['task'] in ENSEMBLE_TASKS:
        if not is_sorted(settings['simulation']['interfaces']):
            msg = "Interface lambda positions in the simulation "\
                "entry are NOT sorted (small to large)"

        if 'ensemble' in settings:
            savelambda = []
            for i_ens, ens in enumerate(settings['ensemble']):
                if 'ensemble_number' not in ens and \
                        'interface' not in ens:
                    msg = "An ensemble has been introduced without "\
                        "references (interface in ensemble settings)"
                else:
                    savelambda.append(settings['simulation']['interfaces']
                                      [i_ens])
                    if 'interface' in ens and ens['interface'] \
                            not in settings['simulation']['interfaces']:
                        msg = "An ensemble has been introduced with an "\
                            "interface not listed in the simulation interfaces"

    if 'msg' in locals():
        raise ValueError(msg)


def check_interfaces(settings):
    """Check that the interfaces are properly defined.

    Parameters
    ----------
    settings : dict
        The current input settings.

    """
    msg = []
    if settings['simulation'].get('flux', False) and \
            not settings['simulation']['zero_ensemble']:
        msg += ['Settings for flux and zero_ensemble are inconsistent.']

    if settings['simulation']['task'] in ENSEMBLE_TASKS:
        if len(settings['simulation']['interfaces']) < 3:
            msg += ['Insufficient number of interfaces for {}'
                    .format(settings['simulation']['task'])]

        if not is_sorted(settings['simulation']['interfaces']):
            msg += ['Interface positions in the simulation interfaces ']
            msg += ['input are NOT properly sorted (ascending order)']

    if msg:
        msgtxt = '\n'.join(msg)
        logger.critical(msgtxt)
        return False

    return True


def is_sorted(lll):
    """Check if a list is sorted."""
    return all(aaa <= bbb for aaa, bbb in zip(lll[:-1], lll[1:]))



# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""This package defines the core PyRETIS tools.

The core tools are intended to define classes which are used in simulations.

Package structure
-----------------

Modules
~~~~~~~

__init__.py
    Import core functions from the modules.

box.py (:py:mod:`pyretis.core.box`)
    Definition of the simulation box class.

common.py (:py:mod:`pyretis.core.common`)
    Some common core methods, for instance for initiating classes.

montecarlo.py (:py:mod:`pyretis.core.montecarlo`)
    This module defines methods for performing Monte Carlo moves.

particlefunctions.py (:py:mod:`pyretis.core.particlefunctions`)
    Functions that operate on (a selection of) particles, for instance
    calculation of the kinetic temperature, pressure, momentum etc.

particles.py (:py:mod:`pyretis.core.particles`)
    Definition of the particle class which is used to represent
    a collection of particles.

pathensemble.py (:py:mod:`pyretis.core.pathensemble`)
    Definition of a class for a collection of paths (i.e. a path
    ensemble).

path.py (:py:mod:`pyretis.core.path`)
    This module defines functions and classes for paths.

properties.py (:py:mod:`pyretis.core.properties`)
    This module defines a class for a generic property.

random_gen.py (:py:mod:`pyretis.core.random_gen`)
    This module defines a class for generating random numbers.

retis.py (:py:mod:`pyretis.core.retis`)
    Module defining methods for performing replica exchange transition
    interface sampling.

system.py (:py:mod:`pyretis.core.system`)
    This module defines the system class which connects different
    parts (for instance box, forcefield and particles) into a single
    structure.

tis.py (:py:mod:`pyretis.core.tis`)
    This module contains methods used in the transition
    interface sampling algorithm.

units.py (:py:mod:`pyretis.core.units`)
    This module defines conversion between units.

Important classes defined in this package
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

BoxBase (:py:class:`.BoxBase`)
    A base class for a simulation box. The box will also
    handle the periodic boundaries.

System (:py:class:`.System`)
    A class which defines the system we are working with. This
    class contain a lot of information and is used to group the
    information into a structure which the simulations will make use
    of. Typically the system will contain a reference to a box,
    a list of particles and also a force field.

Particles (:py:class:`.Particles`)
    A class defining a list of particles. This will contain the
    positions, velocities and forces for the particles.

Path (:py:class:`.Path`)
    A class representing a path. The path contains snapshots with
    some additional information (energies and order parameters).

PathEnsemble (:py:class:`.PathEnsemble`)
    A class representing a collection of paths. The path ensemble
    will not store the full trajectories, but only a simplified
    representation.

PathEnsembleExt (:py:class:`.PathEnsembleExt`)
    A class representing external path ensembles. This handles
    additional bookkeeping related to external paths.

RandomGenerator (:py:class:`.RandomGenerator`)
    A class for generating random numbers.
"""
from .system import System
from .box import create_box
from .particles import Particles, ParticlesExt
from .path import Path
from .pathensemble import PathEnsemble, PathEnsembleExt
from .random_gen import RandomGenerator



# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""Example of performing particle swarm optimization."""
# pylint: disable=invalid-name
import numpy as np
from pyretis.core import create_box, Particles, System
from pyretis.simulation import Simulation
from pyretis.forcefield import ForceField
from psoengine import PSOEngine
from ackley import Ackley, ackley_potential
from matplotlib import pyplot as plt
from matplotlib import animation, cm


NPART = 10
STEPS = 1000
MINX, MAXX = -10, 10
TXT = 'Step: {:5d}: Best: (x, y) = ({:10.3e}, {:10.3e}), pot = {:10.3e}'


def set_up():
    """Just create system and simulation."""
    box = create_box(low=[MINX, MINX], high=[MAXX, MAXX],
                     periodic=[True, True])
    print('Created a box:')
    print(box)

    print(f'Creating system with {NPART} particles')
    system = System(units='reduced', box=box)
    system.particles = Particles(dim=2)
    for _ in range(NPART):
        pos = np.random.uniform(low=MINX, high=MAXX, size=(1, 2))
        system.add_particle(pos)

    ffield = ForceField('Single Ackley function',
                        potential=[Ackley()])
    system.forcefield = ffield
    print(f'Force field is:\n{system.forcefield}')

    print('Creating simulation:')
    engine = PSOEngine(0.7, 1.5, 1.5)
    simulation = Simulation(settings=None, controls={'steps': STEPS})
    task_integrate = {'func': engine.integration_step,
                      'args': [system],
                      'result': 'gbest', 'first': True}
    simulation.add_task(task_integrate)
    return simulation, system


def main():
    """Just run the optimization, no plotting."""
    simulation, _ = set_up()
    for result in simulation.run():
        step = result['cycle']['step']
        best = result['gbest']
        if step % 10 == 0:
            print(TXT.format(step, best[1][0], best[1][1], best[0]))


def evaluate_potential_grid():
    """Evaluate the Ackley potential on a grid."""
    X, Y = np.meshgrid(np.linspace(MINX, MAXX, 100),
                       np.linspace(MINX, MAXX, 100))
    Z = ackley_potential(X, Y)
    return X, Y, Z


def update_animation(frame, system, simulation, scatter):
    """Update animation."""
    patches = []
    if not simulation.is_finished() and frame > 0:
        results = simulation.step()
        best = results['gbest']
        if frame % 10 == 0:
            print(TXT.format(frame, best[1][0], best[1][1], best[0]))
    scatter.set_offsets(system.particles.pos)
    patches.append(scatter)
    return patches


def main_animation():
    """Run the simulation and update for animation."""
    simulation, system = set_up()
    fig = plt.figure()
    ax1 = fig.add_subplot(111, aspect='equal')
    ax1.set_xlim((MINX, MAXX))
    ax1.set_ylim((MINX, MAXX))
    ax1.axes.get_xaxis().set_visible(False)
    ax1.axes.get_yaxis().set_visible(False)
    X, Y, pot = evaluate_potential_grid()
    ax1.contourf(X, Y, pot, cmap=cm.viridis, zorder=1)
    scatter = ax1.scatter(system.particles.pos[:, 0],
                          system.particles.pos[:, 1], marker='o', s=50,
                          edgecolor='#262626', facecolor='white')

    def init():
        """Just return what to re-draw."""
        return [scatter]
    # This will run the animation/simulation:
    anim = animation.FuncAnimation(fig, update_animation,
                                   frames=STEPS+1,
                                   fargs=[system, simulation, scatter],
                                   repeat=False, interval=30, blit=True,
                                   init_func=init)
    plt.show()
    return anim


if __name__ == '__main__':
    main_animation()



# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""A test of the Lennard-Jones pair potential."""
import logging
import unittest
import numpy as np
from pyretis.core.box import create_box
from pyretis.core.particles import Particles
from pyretis.core.system import System
from pyretis.forcefield.potentials.pairpotentials.lennardjones import (
    PairLennardJonesCut,
    PairLennardJonesCutnp,
)
logging.disable(logging.CRITICAL)


CORRECT_FORCE = np.array([[-393024., -2880., 0.],
                          [390144., -390144., 0.],
                          [2880., 393024., 0.]])
CORRECT_VIRIAL = np.array([[196512., 1440., 0.0],
                           [1440., 196512., 0.0],
                           [0.0, 0.0, 0.0]])
CORRECT_VPOT = 32480.0489507


def create_system_box(use_numpy=False):
    """Create a simple test system."""
    box = create_box(cell=[10, 10, 10])
    system = System(box=box)
    system.particles = Particles(system.get_dim())
    system.add_particle(name='Ar', pos=np.array([1.0, 1.0, 1.0]),
                        mass=1.0, ptype=0)
    system.add_particle(name='Ar', pos=np.array([1.5, 1.0, 1.0]),
                        mass=1.0, ptype=0)
    system.add_particle(name='Ar', pos=np.array([1.5, 1.5, 1.0]),
                        mass=1.0, ptype=0)
    if use_numpy:
        pot = PairLennardJonesCutnp()
    else:
        pot = PairLennardJonesCut()
    parameters = {
        0: {'sigma': 1, 'epsilon': 1, 'factor': 2.5},
        1: {'sigma': 2, 'epsilon': 2, 'factor': 2.5},
    }
    pot.set_parameters(parameters)
    return system, box, pot


class TestLennardJonesCutPairPotential(unittest.TestCase):
    """Run tests for the Lennard-Jones pair potential."""

    def test_parameters(self):
        """Test setup with parameters."""
        pot = PairLennardJonesCut()
        parameters = {
            0: {'sigma': 1, 'epsilon': 1, 'factor': 2.5},
            1: {'sigma': 2, 'epsilon': 2, 'factor': 2.5},
        }
        pot.set_parameters(parameters)
        correct_pair = {
            (0, 0): {'epsilon': 1, 'rcut': 2.5, 'sigma': 1.},
            (0, 1): {'epsilon': np.sqrt(2.), 'rcut': 2.5 * np.sqrt(2.),
                     'sigma': np.sqrt(2.)},
            (1, 1): {'sigma': 2, 'epsilon': 2, 'rcut': 2.5 * 2.},
        }
        correct_pair[(1, 0)] = correct_pair[(0, 1)]
        for key, val in correct_pair.items():
            for key2, val2 in val.items():
                self.assertAlmostEqual(val2, pot.params[key][key2])
        parameters = {
            0: {'sigma': 1, 'epsilon': 1, 'factor': 0.0},
        }
        pot.set_parameters(parameters)
        self.assertAlmostEqual(pot.params[(0, 0)]['rcut'], 0.0)

    def test_potential(self):
        """Test evaluation of the Lennard-Jones potential."""
        system, _, pot = create_system_box()
        vpot = pot.potential(system)
        self.assertAlmostEqual(vpot, CORRECT_VPOT)

    def test_force(self):
        """Test evaluation of the Lennard-Jones force."""
        system, _, pot = create_system_box()
        force, virial = pot.force(system)
        self.assertTrue(np.allclose(force, CORRECT_FORCE))
        self.assertTrue(np.allclose(virial, CORRECT_VIRIAL))

    def test_potential_and_force(self):
        """Test evaluate of Lennard-Jones potential and force."""
        system, _, pot = create_system_box()
        vpot, force, virial = pot.potential_and_force(system)
        self.assertTrue(np.allclose(force, CORRECT_FORCE))
        self.assertTrue(np.allclose(virial, CORRECT_VIRIAL))
        self.assertAlmostEqual(vpot, CORRECT_VPOT)


class TestLennardJonesCutPairPotentialnp(unittest.TestCase):
    """Run tests for the Lennard-Jones pair potential (numpy)."""

    def test_potential(self):
        """Test evaluation of the Lennard-Jones potential (numpy)."""
        system, _, pot = create_system_box(use_numpy=True)
        vpot = pot.potential(system)
        self.assertAlmostEqual(vpot, CORRECT_VPOT)

    def test_force(self):
        """Test evaluation of the Lennard-Jones force (numpy)."""
        system, _, pot = create_system_box(use_numpy=True)
        force, virial = pot.force(system)
        self.assertTrue(np.allclose(force, CORRECT_FORCE))
        self.assertTrue(np.allclose(virial, CORRECT_VIRIAL))

    def test_potential_and_force(self):
        """Test evaluate of Lennard-Jones potential and force (numpy)."""
        system, _, pot = create_system_box(use_numpy=True)
        vpot, force, virial = pot.potential_and_force(system)
        self.assertTrue(np.allclose(force, CORRECT_FORCE))
        self.assertTrue(np.allclose(virial, CORRECT_VIRIAL))
        self.assertAlmostEqual(vpot, CORRECT_VPOT)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""Test the PathStorage classes."""
import logging
import os
import unittest
import tempfile
import tarfile
import pathlib
from pyretis.inout.common import make_dirs
from pyretis.core.pathensemble import PathEnsemble
from pyretis.inout.archive import (
    add_to_tar_file,
    generate_traj_names,
    PathStorage,
    PathStorageTar,
)
from pyretis.inout.formats import (
    OrderPathFormatter,
    EnergyPathFormatter,
    PathExtFormatter,
)
from .help import create_external_path, turn_on_logging

logging.disable(logging.CRITICAL)
HERE = os.path.abspath(os.path.dirname(__file__))


def create_dirs_and_files(path_ensemble, path):
    """Create some directories and files for the testing."""
    for i in path_ensemble.directories():
        make_dirs(i)
    new_pos = []
    for i in path.phasepoints:
        filename = os.path.join(
            path_ensemble.directory['generate'], i.particles.get_pos()[0]
        )
        new_pos.append((filename, i.particles.get_pos()[1]))
        pathlib.Path(filename).touch()
    for phasepoint, pos in zip(path.phasepoints, new_pos):
        phasepoint.particles.set_pos(pos)


def generate_file_names(start, stop, dirname, paths):
    """Just generate some fake file names."""
    files = []
    for i in range(start, stop):
        filename = os.path.join(dirname, f'file-{i}.txt')
        pathlib.Path(filename).touch()
        files.append(
            (filename, os.path.join(*paths, f'file-{i}.txt'))
        )
    return files


def look_for_files_and_dirs(dirname):
    """Return files or directories found in the given path."""
    files = []
    dirs = []
    for i in os.scandir(dirname):
        if i.is_file():
            files.append(i.name)
        elif i.is_dir():
            dirs.append(i.name)
    return files, dirs


class TestArchiveMethods(unittest.TestCase):
    """Test methods defined in the module."""

    def test_add_to_tar(self):
        """Test adding files."""
        with tempfile.TemporaryDirectory() as tempdir:
            tar_file = os.path.join(tempdir, 'test.tar')
            # Create some files to add:
            files = generate_file_names(0, 5, tempdir, ['aaa', 'bbb', 'ccc'])
            add = add_to_tar_file(tar_file, files, file_mode='w')
            self.assertTrue(add)
            with tarfile.open(tar_file, 'r') as tar:
                members = [i.name for i in tar.getmembers()]
                self.assertEqual(len(members), len(files))
                for _, i in files:
                    self.assertIn(i, members)
            # Append to the file:
            files2 = generate_file_names(5, 10, tempdir, ['123', '456', '689'])
            add = add_to_tar_file(tar_file, files2, file_mode='a')
            self.assertTrue(add)
            files += files2
            with tarfile.open(tar_file, 'r') as tar:
                members = [i.name for i in tar.getmembers()]
                self.assertEqual(len(members), len(files))
                for (_, i) in files:
                    self.assertIn(i, members)
            # Check if we can get an read error:
            with open(tar_file, 'w', encoding='utf-8') as tar:
                tar.write('Some Men Just Want to Watch The World Burn')
            add = add_to_tar_file(tar_file, files, file_mode='a')
            self.assertFalse(add)

    def test_generate_names(self):
        """Test the generation of trajectory names for the archive."""
        path, _ = create_external_path()
        files = generate_traj_names(path, 'traj')
        for (src, trg) in files:
            self.assertEqual(
                trg,
                os.path.join('traj', src)
            )


class TestPathStorageTar(unittest.TestCase):
    """Test the PathStorageTar class."""

    def test_output(self):
        """Test that we can output to the storage."""
        storage = PathStorageTar()
        path, _ = create_external_path()
        with tempfile.TemporaryDirectory() as tempdir:
            path_ensemble = PathEnsemble(1, [0.0, 1.0, 2.0], exe_dir=tempdir)
            create_dirs_and_files(path_ensemble, path)
            for j, status in enumerate(('ACC', 'REJ')):
                files = storage.output(10 + j, (path, status, path_ensemble))
                # Check if we got the files in the tar-files:
                archive = storage.archive_name_from_status(status)
                tar_file = os.path.join(path_ensemble.directory['traj'],
                                        archive)
                with tarfile.open(tar_file, 'r') as tar:
                    members = [i.name for i in tar.getmembers()]
                    self.assertEqual(len(members), len(files))
                    for _, i in files:
                        self.assertIn(i, members)

    def test_output_recreate(self):
        """Test that the backup of archives does work."""
        storage = PathStorageTar()
        path, _ = create_external_path()
        with tempfile.TemporaryDirectory() as tempdir:
            path_ensemble = PathEnsemble(1, [0.0, 1.0, 2.0], exe_dir=tempdir)
            create_dirs_and_files(path_ensemble, path)
            open(os.path.join(tempdir,
                              '001',
                              'ensemble.restart'), 'a',
                 encoding='utf-8').close()
            storage.output(10, (path, 'ACC', path_ensemble))
            tar_file = os.path.join(path_ensemble.directory['traj'],
                                    storage.archive_name_from_status('ACC'))
            # This will overwrite/corrupt the tar archive:
            with open(tar_file, 'w', encoding='utf-8') as tar:
                tar.write('Some Men Just Want to Watch The World Burn')
            # We should be able to create a new archive and backup the old:
            storage.output(11, (path, 'ACC', path_ensemble))
            expected = [
                storage.archive_name_from_status('ACC'),
                f"{storage.archive_name_from_status('ACC')}_000"
            ]
            files, _ = look_for_files_and_dirs(path_ensemble.directory['traj'])
            self.assertEqual(len(expected), len(files))
            for i in expected:
                self.assertIn(i, files)


class TestPathStorage(unittest.TestCase):
    """Test the PathStorage class."""

    def test_write_fail(self):
        """Test that the write method give an critical log message."""
        storage = PathStorage()
        with turn_on_logging():
            with self.assertLogs('pyretis.inout.archive', level='CRITICAL'):
                storage.write('test')

    def test_formatter_info(self):
        """Test that we get correct info about formatters."""
        storage = PathStorage()
        info = storage.formatter_info()
        correct = [OrderPathFormatter, EnergyPathFormatter, PathExtFormatter]
        self.assertEqual(len(info), len(correct))
        for i in correct:
            self.assertIn(i, info)

    def test_output(self):
        """Test that we can create output."""
        storage = PathStorage()
        path, _ = create_external_path()
        with tempfile.TemporaryDirectory() as tempdir:
            path_ensemble = PathEnsemble(1, [0.0, 1.0, 2.0], exe_dir=tempdir)
            create_dirs_and_files(path_ensemble, path)
            # Now, there should be no output yet:
            files, dirs = look_for_files_and_dirs(
                path_ensemble.directory['traj'])
            self.assertEqual(len(files), 0)
            self.assertEqual(len(dirs), 0)
            # Output some trajectories:
            statuses = ('ACC', 'REJ', 'AN USER ERROR')
            files = {}
            for i, status in enumerate(statuses):
                open(os.path.join(tempdir,
                                  '001',
                                  'ensemble.restart'), 'a',
                     encoding='utf-8').close()
                written = storage.output(10 + i, (path, status, path_ensemble))
                target_dir = os.path.join(
                    storage.archive_name_from_status(status),
                    storage.out_dir_fmt.format(10 + i),
                )
                # Grab path to the files added:
                files[target_dir] = [j[0] for j in written]
            # We now expect to have some files:
            for key, val in files.items():
                dir_name = os.path.join(path_ensemble.directory['traj'], key)
                self.assertTrue(os.path.isdir(dir_name))
                # Check that the files we claimed to make are present
                # as files:
                for i in val:
                    self.assertTrue(os.path.isfile(i))
                # Just search for all files in the given archive folder
                # to check that we did not create anything extra:
                found_files = []
                for root, _, filei in os.walk(dir_name):
                    found_files += [os.path.join(root, i) for i in filei]
                self.assertEqual(
                    len(val), len(found_files)
                )
                for i in found_files:
                    self.assertIn(i, val)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""Example of running a MD NVE simulation.

In this example, we animate the output.
"""
# pylint: disable=invalid-name
import numpy as np
# imports for the plotting:
from matplotlib import pyplot as plt
from matplotlib import animation
import matplotlib as mpl
from pyretis.core.units import CONVERT, create_conversion_factors
from pyretis.inout.plotting import COLORS, COLOR_SCHEME
from pyretis.setup import create_simulation

ljparams = {0: {'sigma': 1.0, 'epsilon': 1.0, 'rcut': 2.5}}
# Define simulation settings:
settings = {}
settings['system'] = {
    'temperature': 1.0,
    'dimensions': 2,
    'units': 'lj'
}
settings['box'] = {
    'low': [0.0, 0.0],
    'high': [1.1*3.405, 1.1*3.405],
    'periodic': [True, True]
}
settings['simulation'] = {
    'task': 'md-nve',
    'steps': 950
}
settings['engine'] = {
    'class': 'velocityverlet',
    'timestep': 0.0025
}
settings['output'] = {
    'backup': 'overwrite',
    'energy-file': 1,
    'screen': 10,
    'trajectory-file': 10
}
settings['potential'] = [
    {'class': 'PairLennardJonesCutnp',
     'dim': 2, 'shift': True,
     'mixing': 'geometric',
     'parameter': ljparams}
]
settings['particles'] = {
    'position': {'input_file': 'initial.xyz'},
    'velocity': {'generate': 'maxwell', 'momentum': True, 'seed': 0}
}
UNIT = settings['system']['units']
create_conversion_factors(UNIT)
SIGMA = CONVERT['length'][UNIT, 'A']
ECONV = CONVERT['energy'][UNIT, 'kcal/mol']
print('# Creating system from settings.')
simulation_nve = create_simulation(settings)
print('# Creating output tasks from settings.')
simulation_nve.set_up_output(settings, progress=False)
ljsystem = simulation_nve.system
size = ljsystem.box.bounds()
npart = ljsystem.particles.npart
msg = 'Added {:d} particles to a simple square lattice'
print(msg.format(npart))
npart = float(npart)

# We will in this example animate on the fly.
# We will then have to do some extra set up. The actual simulation is
# carried out by calling `simulation_nve.step()` in the `update`
# function which is executed by the animation.FuncAnimation() call.
# In effect animation.FuncAnimation will run the simulation one step,
# update the plot and display it and continue this loop until the
# simulation is done.
timeunit = (settings['engine']['timestep'] *
            CONVERT['time'][UNIT, 'fs'])
timeendfs = settings['simulation']['steps'] * timeunit
time, step, v_pot, e_kin, e_tot, temperature = [], [], [], [], [], []
mpl.rc('axes', labelsize='large')
mpl.rc('font', family='serif')
fig = plt.figure(figsize=(12, 6))
# This adds the first axis. Here we will plot the
# particles with velocity and force vectors.
ax1 = fig.add_subplot(121)
ax1.set_xlim((size[0] + np.array([-0.5, 0.5])) * SIGMA)
ax1.set_ylim((size[1] + np.array([-0.5, 0.5])) * SIGMA)
ax1.set_aspect('equal', 'datalim')
ax1.set_xlabel(u'x / Å')
ax1.set_ylabel(u'y / Å')
ax1.set_xticks([size[0][0] * SIGMA, size[0][1] * SIGMA])
ax1.set_yticks([size[1][0] * SIGMA, size[1][1] * SIGMA])
ax1.xaxis.labelpad = -5
ax1.yaxis.labelpad = -5

pos0 = ljsystem.box.pbc_wrap(ljsystem.particles.pos)
# Set up circles to represent the particles:
circles = []
for _ in range(int(npart)):
    circles.append(plt.Circle((0, 0), radius=SIGMA * 0.5, alpha=0.5,
                              color='blue'))
    circles[-1].set_visible(False)
    ax1.add_patch(circles[-1])
# Add arrows for the forces and velocities:
force_arrow = plt.quiver(pos0[:, 0], pos0[:, 1],
                         color=COLORS['almost_black'], zorder=4)
vel_arrow = plt.quiver(pos0[:, 0], pos0[:, 1],
                       color=COLOR_SCHEME['colorblind_10'][1], zorder=4)
# Also add arrows for a "legend":
plt.quiverkey(force_arrow, 3, -3.5, 9, 'Forces', coordinates='data',
              color=COLORS['almost_black'], fontproperties={'size': 'large'})
plt.quiverkey(vel_arrow, 9, -3.5, 9, 'Velocities', coordinates='data',
              color=COLOR_SCHEME['colorblind_10'][1],
              fontproperties={'size': 'large'})
# Draw the lines representing the box boundaries:
ax1.axhline(y=size[1][0] * SIGMA, lw=4, ls=':', alpha=0.5,
            color=COLORS['almost_black'])
ax1.axhline(y=size[1][1] * SIGMA, lw=4, ls=':', alpha=0.5,
            color=COLORS['almost_black'])
ax1.axvline(x=size[0][0] * SIGMA, lw=4, ls=':', alpha=0.5,
            color=COLORS['almost_black'])
ax1.axvline(x=size[0][1] * SIGMA, lw=4, ls=':', alpha=0.5,
            color=COLORS['almost_black'])
# Add second axis for plotting the energies
ax2 = fig.add_subplot(122)
ax2.set_xlim(0, timeendfs)
ax2.set_ylim(-0.05, 0.25)
ax2.set_xlabel('Time / fs')
ax2.set_ylabel('Energy / (kcal/mol)')
time_text = ax2.text(0.02, 0.95, '', transform=ax2.transAxes)
linepot, = ax2.plot([], [], lw=4, ls='-', color=COLOR_SCHEME['deep'][0],
                    alpha=0.8, label='Potential')
linekin, = ax2.plot([], [], lw=4, ls='-', color=COLOR_SCHEME['deep'][1],
                    alpha=0.8, label='Kinetic')
linetot, = ax2.plot([], [], lw=4, ls='-', color=COLORS['almost_black'],
                    alpha=0.8, label='Total')
ax2.legend(loc='lower left', ncol=2, frameon=False)
plt.subplots_adjust(left=0.08, right=0.97, top=0.95, bottom=0.15)


def get_max_vector(vectors):
    """Determine the longest vector in a list of vectors.

    Parameters
    ----------
    vectors : numpy.array
        Numpy array of vectors to analyse, using `numpy.dot`.

    Returns
    -------
    vmax : float
        The length of the largest vector.

    """
    vmax = None
    for vi in vectors:
        vs = np.sqrt(np.dot(vi, vi))
        if vmax is None or vs > vmax:
            vmax = vs
    return vmax


def get_velocity_force_arrows(forces, vels):
    """Obtain the force and velocity vectors.

    Parameters
    ----------
    forces : numpy.array
        The forces on the particles.
    vels : numpy.array
        The velocity of the particles.

    Returns
    -------
    out[0] : numpy.array
        The x-component of the forces, normalised.
    out[1] : numpy.array
        The y-component of the forces, normalised.
    out[2] : numpy.array
        The x-component of the velocities, normalised.
    out[3] : numpy.array
        The y-component of the velocities, normalised.

    """
    fmax, vmax = get_max_vector(forces), get_max_vector(vels)
    forceu, forcev, velu, velv = [], [], [], []
    for fi, vi in zip(forces, vels):
        fj = 10.0 * fi / fmax
        vj = 10.0 * vi / vmax
        forceu.append(fj[0])
        forcev.append(fj[1])
        velu.append(vj[0])
        velv.append(vj[1])
    return forceu, forcev, velu, velv


def update(frame, sim):
    """Update plots for the animation.

    This function will be running the simulation and updating the plots.
    It is called one time per step, and we choose to update the simulation
    inside this function

    Parameters
    ----------
    frame : int
        The current frame number, supplied by `animation.FuncAnimation`.
    sim : object like `Simulation`
        The simulation we are running.

    Returns
    -------
    out : list
        list of the patches to be drawn.

    """
    particles = sim.system.particles
    pos = sim.system.box.pbc_wrap(particles.pos)
    patches = []
    # update positions of the circles according to the particles:
    for ci, pi in zip(circles, pos):
        ci.center = np.array([pi[0], pi[1]]) * SIGMA
        ci.set_visible(True)
        patches.append(ci)
    # update the force and velocity vectors:
    forceu, forcev, velu, velv = get_velocity_force_arrows(particles.force,
                                                           particles.vel)
    force_arrow.set_offsets(pos * SIGMA)
    force_arrow.set_UVC(forceu, forcev)
    force_arrow.set_visible(True)
    patches.append(force_arrow)
    vel_arrow.set_offsets(pos * SIGMA)
    vel_arrow.set_UVC(velu, velv)
    vel_arrow.set_visible(True)
    patches.append(vel_arrow)

    if not sim.is_finished():
        result = sim.step()
        for tsk in sim.output_tasks:
            tsk.output(result)
        # here we calculate some energies and updates the energy plots:
        step.append(result['cycle']['step'])
        time.append(step[-1] * timeunit)
        temperature.append(result['thermo']['temp'])
        v_pot.append(ECONV * result['thermo']['vpot'])
        e_kin.append(ECONV * result['thermo']['ekin'])
        e_tot.append(e_kin[-1] + v_pot[-1])
        # update plots with energies:
        linepot.set_data(time, (np.array(v_pot) - v_pot[0]))
        patches.append(linepot)
        linekin.set_data(time, (np.array(e_kin)))
        patches.append(linekin)
        linetot.set_data(time, (np.array(e_tot) - v_pot[0]))
        patches.append(linetot)
        # also display current simulation time;
        time_text.set_text(f'Time: {time[-1]:6.2f} fs (frame: {frame})')
        patches.append(time_text)
        return patches
    print('Simulation is done.')
    return patches


def init():
    """Declare what to re-draw when clearing the animation frame.

    Returns
    -------
    out : list
        list of the patches to be drawn.

    """
    patches = []
    force_arrow.set_visible(False)
    patches.append(force_arrow)
    vel_arrow.set_visible(False)
    patches.append(vel_arrow)
    time_text.set_text('')
    patches.append(time_text)
    for ci in circles:
        ci.set_visible(False)
        patches.append(ci)
    return patches


# This will run the animation/simulation:
anim = animation.FuncAnimation(fig, update,
                               frames=settings['simulation']['steps']+1,
                               fargs=[simulation_nve],
                               repeat=False, interval=2, blit=True,
                               init_func=init)
# for making a movie:
# anim.save('particles.mp4', fps=30, extra_args=['-vcodec', 'libx264'])
plt.show()



# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""Module defining Lennard-Jones pair potentials.

This module defines the Lennard-Jones potential for PyRETIS.

Important classes defined here
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

PairLennardJonesCut (:py:class:`.PairLennardJonesCut`)
    A class representing a Lennard-Jones 6-12 potential implemented
    in pure python.

PairLennardJonesCutnp (:py:class:`.PairLennardJonesCutnp`)
    A class representing a Lennard-Jones 6-12 potential implemented
    using numpy.
"""
import logging
import numpy as np
from pyretis.forcefield.potential import PotentialFunction
from .pairpotential import generate_pair_interactions


logger = logging.getLogger(__name__)  # pylint: disable=invalid-name
logger.addHandler(logging.NullHandler())


__all__ = ['PairLennardJonesCut', 'PairLennardJonesCutnp']


class PairLennardJonesCut(PotentialFunction):
    r"""Lennard-Jones 6-12 potential in pure Python.

    This class implements as simple Lennard-Jones 6-12 potential which
    employs a simple cut-off and can be shifted. The potential energy
    (:math:`V_\text{pot}`) is defined in the usual way for an
    interacting pair of particles a distance :math:`r` apart,

    .. math::

       V_\text{pot} = 4 \varepsilon \left( x^{12} - x^{6} \right),

    where :math:`x = \sigma/r` and :math:`\varepsilon`
    and :math:`\sigma` are the potential parameters. The parameters are
    stored as attributes of the potential, we store one set for each
    kind of pair interaction. Parameters can be generated with a
    specific mixing rule by the force field.

    This implementation is in pure python (yes we are double looping!)
    and it is slow. It should not be used for production, please
    consider the numpy aware `PairLennardJonesCutnp` instead.

    Attributes
    ----------
    params : dict
        The parameters for the potential. This dict is assumed to
        contain parameters for pairs, i.e. for interactions.
    _lj : dict of dict
        Lennard-Jones parameters used for calculation of the force.
        Keys are the pairs (particle types) that may interact.
          [1] Calculated as: ``48.0 * epsilon * sigma**12``
          [2] Calculated as: ``24.0 * epsilon * sigma**6``
          [3] Calculated as: ``4.0 * epsilon * sigma**12``
          [4] Calculated as: ``4.0 * epsilon * sigma**6``
    _offset : dict
        Potential values for shifting the potential if requested.
        This is the potential evaluated at the cut-off.
    _rcut2 : dict
        The squared cut-off for each interaction type.
        Keys are the pairs (particle types) that may interact.

    """

    def __init__(self, dim=3, shift=True, mixing='geometric',
                 desc='Lennard-Jones pair potential'):
        """Initialise the Lennard-Jones potential.

        Parameters
        ----------
        dim : int, optional
            The dimensionality to use.
        shift : boolean, optional
            Determines if the potential should be shifted or not.
        mixing : string, optional
            Determines how we should mix potential parameters.
        desc : string, optional
            Description of the potential.

        """
        super().__init__(dim=dim, desc=desc)
        self.shift = shift
        self._lj = {1: {}, 2: {}, 3: {}, 4: {}}
        self._rcut2 = {}
        self._offset = {}
        self.params = {}
        self.mixing = mixing

    def set_parameters(self, parameters):
        """Update all parameters.

        Here, we generate pair interactions, since that is what this
        potential actually is using.

        Parameters
        ----------
        parameters : dict
            The input pair parameters.

        """
        self.params = {}
        pair_param = generate_pair_interactions(parameters, self.mixing)
        for pair, item in pair_param.items():
            eps_ij = item['epsilon']
            sig_ij = item['sigma']
            rcut = item['rcut']
            self._lj[1][pair] = 48.0 * eps_ij * sig_ij**12
            self._lj[2][pair] = 24.0 * eps_ij * sig_ij**6
            self._lj[3][pair] = 4.0 * eps_ij * sig_ij**12
            self._lj[4][pair] = 4.0 * eps_ij * sig_ij**6
            self._rcut2[pair] = rcut**2
            vcut = 0.0
            if self.shift:
                try:
                    vcut = 4.0 * eps_ij * ((sig_ij / rcut)**12 -
                                           (sig_ij / rcut)**6)
                except ZeroDivisionError:
                    vcut = 0.0
            self._offset[pair] = vcut
            self.params[pair] = item

    def __str__(self):
        """Generate a string with the potential parameters.

        It will generate a string with both pair and atom parameters.

        Returns
        -------
        out : string
            Table with the parameters of all interactions.

        """
        strparam = [self.desc]
        strparam += ['Potential parameters, Lennard-Jones:']
        useshift = 'yes' if self.shift else 'no'
        strparam.append(f'Shift potential: {useshift}')
        atmformat = '{0:12s} {1:>9s} {2:>9s} {3:>9s}'
        atmformat2 = '{0:12s} {1:>9.4f} {2:>9.4f} {3:>9.4f}'
        strparam.append('Pair parameters:')
        strparam.append(atmformat.format('Atom/pair', 'epsilon', 'sigma',
                                         'cut-off'))
        for pair in sorted(self.params):
            eps_ij = self.params[pair]['epsilon']
            sig_ij = self.params[pair]['sigma']
            rcut = np.sqrt(self._rcut2[pair])
            stri = f'{pair[0]}-{pair[1]}'
            strparam.append(atmformat2.format(stri, eps_ij, sig_ij, rcut))
        return '\n'.join(strparam)

    def potential(self, system):
        """Calculate the potential energy for the Lennard-Jones interaction.

        Parameters
        ----------
        system : object like :py:class:`.System`
            The system for which we calculate the potential.

        Returns
        -------
        The potential energy as a float.

        """
        particles = system.particles
        box = system.box
        v_pot = 0.0
        for pair in particles.pairs():
            i, j, itype, jtype = pair
            delta = box.pbc_dist_coordinate(particles.pos[i] -
                                            particles.pos[j])
            rsq = np.dot(delta, delta)
            if rsq < self._rcut2[itype, jtype]:
                r2inv = 1.0/rsq
                r6inv = r2inv**3
                v_pot += (r6inv * (self._lj[3][itype, jtype] * r6inv -
                                   self._lj[4][itype, jtype]) -
                          self._offset[itype, jtype])
        return v_pot

    def force(self, system):
        """Calculate the force for the Lennard-Jones interaction.

        We also calculate the virial here, since the force
        is evaluated.

        Parameters
        ----------
        system : object like :py:class:`.System`
            The system for which we calculate the force.

        Returns
        -------
        out[0] : numpy.array
            The force as a numpy.array.
        out[1] : numpy.array
            The virial as a numpy.array.

        """
        particles = system.particles
        forces = np.zeros(particles.pos.shape)
        virial = np.zeros((system.box.dim, system.box.dim))
        for pair in particles.pairs():
            i, j, itype, jtype = pair
            delta = system.box.pbc_dist_coordinate(particles.pos[i] -
                                                   particles.pos[j])
            if np.dot(delta, delta) < self._rcut2[itype, jtype]:
                r2inv = 1.0 / np.dot(delta, delta)
                r6inv = r2inv**3
                forcelj = r2inv * r6inv * (self._lj[1][itype, jtype] * r6inv -
                                           self._lj[2][itype, jtype])
                forceij = forcelj * delta
                forces[i] += forceij
                forces[j] -= forceij
                virial += np.outer(forceij, delta)
        return forces, virial

    def potential_and_force(self, system):
        """Calculate potential and force for the Lennard-Jones interaction.

        Since the force is evaluated, the virial is also calculated.

        Parameters
        ----------
        system : object like :py:class:`.System`
            The system for which we calculate the potential and force.

        Note
        ----
        Currently, the virial is only calculated for all the particles.
        It is not calculated as per atom virial. The virial
        per atom might be useful to obtain a local pressure or stress,
        however, this needs some consideration. Perhaps it's best to
        fully implement this as a method of planes or something similar.

        Returns
        -------
        out[0] : float
            The potential energy as a float.
        out[1] : numpy.array
            The force as a numpy.array of the same shape as the
            positions in `particles.pos`.
        out[2] : numpy.array
            The virial, as a symmetric matrix with dimensions
            (dim, dim) where dim is given by the box/system dimensions.

        """
        v_pot = 0.0
        forces = np.zeros(system.particles.pos.shape)
        virial = np.zeros((system.box.dim, system.box.dim))
        for pair in system.particles.pairs():
            i, j, itype, jtype = pair
            delta = system.box.pbc_dist_coordinate(system.particles.pos[i] -
                                                   system.particles.pos[j])
            rsq = np.dot(delta, delta)
            if rsq < self._rcut2[itype, jtype]:
                r2inv = 1.0 / rsq
                r6inv = r2inv**3
                v_pot += (r6inv * (self._lj[3][itype, jtype] * r6inv -
                                   self._lj[4][itype, jtype]) -
                          self._offset[itype, jtype])
                forceij = (delta * r2inv * r6inv *
                           (self._lj[1][itype, jtype] * r6inv -
                            self._lj[2][itype, jtype]))
                forces[i] += forceij
                forces[j] -= forceij
                virial += np.outer(forceij, delta)
        return v_pot, forces, virial


class PairLennardJonesCutnp(PairLennardJonesCut):
    """Lennard-Jones 6-12 potential with numpy.

    A Lennard-Jones 6-12 potential with a simple cut-off which can be
    shifted. `PairLennardJonesCutnp` uses numpy for calculations, i.e.
    most operations are recast as numpy.array operations. Otherwise, it
    is similar to `PairLennardJonesCut`.
    """

    def __init__(self, dim=3, shift=True, mixing='geometric',
                 desc='Lennard-Jones pair potential (numpy)'):
        """Initialise the Lennard-Jones potential.

        Parameters
        ----------
        dim : int, optional
            The dimensionality to use.
        shift : boolean, optional
            Determines if the potential should be shifted or not.
        mixing : string, optional
            Describes the mixing rules for the parameters.
        desc : string, optional
            Description of the potential.

        """
        super().__init__(dim=dim, desc=desc,
                         shift=shift, mixing=mixing)

    def potential(self, system):
        """Calculate the potential energy for the Lennard-Jones interaction.

        Parameters
        ----------
        system : object like :py:class:`.System`
            The system for which we calculate the potential.

        Returns
        -------
        out : float
            The potential energy as a float.

        """
        particles = system.particles
        box = system.box
        pot = 0.0
        # the particle list may implement a list which we can
        # loop over. This could be some kind of fancy neighbour list
        # here, we ignore this and loop over all pairs using numpy.
        for i, particle_i in enumerate(particles.pos[:-1]):
            itype = particles.ptype[i]
            delta = particle_i - particles.pos[i+1:]
            delta = box.pbc_dist_matrix(delta)
            rsq = np.einsum('ij, ij->i', delta, delta)
            k = np.where(_check_cutoff(self._rcut2, rsq,
                                       particles.ptype[i+1:],
                                       itype))[0]
            if len(k) > 0:  # pylint: disable=len-as-condition
                r6inv = 1.0 / rsq[k]**3
                pot += np.sum(_pot_term(self._lj, self._offset,
                                        r6inv, particles.ptype[k+i+1], itype))
        return pot

    def force(self, system):
        """Calculate the force for the Lennard-Jones interaction.

        We also calculate the virial here, since the force
        is evaluated.

        Parameters
        ----------
        system : object like :py:class:`.System`
            The system for which we calculate the force.

        Note
        ----
        The way the "dim" is used may be reconsidered. There is
        already a self.dim parameter for the potential class.

        Returns
        -------
        out[0] : numpy.array
            The force as a numpy.array of the same shape as the
            positions in particles.pos.
        out[1] : numpy.array
            The virial, as a symmetric matrix with dimensions (dim, dim)
            where dim is given by the box.

        """
        particles = system.particles
        forces = np.zeros(particles.pos.shape)
        virial = np.zeros((system.box.dim, system.box.dim))
        for i, particle_i in enumerate(particles.pos[:-1]):
            itype = particles.ptype[i]
            delta = particle_i - particles.pos[i+1:]
            delta = system.box.pbc_dist_matrix(delta)
            rsq = np.einsum('ij, ij->i', delta, delta)
            k = np.where(_check_cutoff(self._rcut2, rsq,
                                       particles.ptype[i+1:],
                                       itype))[0]
            if len(k) > 0:  # pylint: disable=len-as-condition
                r2inv = 1.0 / rsq[k]
                r6inv = r2inv**3
                forcelj = _force_term(self._lj, r2inv, r6inv,
                                      particles.ptype[k+i+1], itype)
                forceij = np.einsum('i,ij->ij', forcelj, delta[k])
                forces[i] += np.sum(forceij, axis=0)
                forces[k+i+1] -= forceij
                virial += np.einsum('ij,ik->jk', forceij, delta[k])
        return forces, virial

    def potential_and_force(self, system):
        """Calculate the potential & force for the Lennard-Jones interaction.

        We also calculate the virial here, since the force is evaluated.

        Parameters
        ----------
        system : object like :py:class:`.System`
            The system for which we calculate the potential and force.

        Note
        ----
        Currently, the virial is only calculated for all the particles.
        It is not calculated as a per atom virial. The virial per
        atom might be useful to obtain a local pressure or stress,
        however, this needs some consideration. Perhaps it's best to
        fully implement this as a method of planes or something similar.

        Returns
        -------
        out[0] : float
            The potential energy as a float.
        out[1] : numpy.array
            The force as a numpy.array of the same shape as the
            positions in `particles.pos`.
        out[2] : numpy.array
            The virial, as a symmetric matrix with dimensions (dim, dim)
            where dim is given by the box.

        """
        particles = system.particles
        pot = 0.0
        forces = np.zeros(particles.pos.shape)
        virial = np.zeros((system.box.dim, system.box.dim))
        for i, particle_i in enumerate(particles.pos[:-1]):
            delta = particle_i - particles.pos[i+1:]
            delta = system.box.pbc_dist_matrix(delta)
            rsq = np.einsum('ij, ij->i', delta, delta)
            k = np.where(_check_cutoff(self._rcut2, rsq,
                                       particles.ptype[i+1:],
                                       particles.ptype[i]))[0]
            if len(k) > 0:  # pylint: disable=len-as-condition
                r2inv = 1.0 / rsq[k]
                r6inv = r2inv**3
                pot += np.sum(_pot_term(self._lj, self._offset,
                                        r6inv, particles.ptype[k+i+1],
                                        particles.ptype[i]))
                forcelj = _force_term(self._lj, r2inv, r6inv,
                                      particles.ptype[k+i+1],
                                      particles.ptype[i])
                forceij = np.einsum('i,ij->ij', forcelj, delta[k])
                forces[i] += np.sum(forceij, axis=0)
                forces[k+i+1] -= forceij
                virial += np.einsum('ij,ik->jk', forceij, delta[k])
        return pot, forces, virial


@np.vectorize
def _pot_term(llj, offset, r6inv, itype, jtype):
    """Lennard Jones potential term."""
    return (r6inv * (llj[3][itype, jtype] * r6inv - llj[4][itype, jtype])
            - offset[itype, jtype])


@np.vectorize
def _force_term(llj, r2inv, r6inv, jtype, itype):
    """Lennard Jones force term."""
    return r2inv * r6inv * (llj[1][itype, jtype] * r6inv -
                            llj[2][itype, jtype])


@np.vectorize
def _check_cutoff(rcut2, rsq, jtype, itype):
    """Check if we are closer than the cut-off."""
    return rsq < rcut2[itype, jtype]



# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""A test using the LAMMPS engine."""
import math
import os
import colorama
import numpy as np
from pyretis.core.random_gen import create_random_generator
from pyretis.inout.common import make_dirs
from pyretis.engines.lammps import LAMMPSEngine
from pyretis.inout import print_to_screen
from pyretis.testing.systemhelp import create_system_ext
from pyretis.testing.helpers import clean_dir


HERE = os.path.abspath(os.path.dirname(__file__))


def backup_files(dirname, prefix='prev_'):
    """Backup some files so we can inspect them later."""
    files = {'generate_vel.in', 'generate_vel.lammpstrj',
             'generate_vel.log', 'generate_vel.screen',
             'order_generate_vel.txt'}
    backup = {}
    for i in files:
        new_name = os.path.join(dirname, f'{prefix}{i}')
        os.rename(os.path.join(dirname, i), new_name)
        backup[i] = new_name
    return backup


def modify_velocities():
    """Use the LAMMPS engine to run a MD simulation forward in time."""
    print_to_screen('\nTesting that we can draw randomized velocities.')
    engine = LAMMPSEngine('lmp_serial', 'lammps_input', 1)
    # Create a dummy system:
    system = create_system_ext(pos=('system.data', 0))
    exe_dir = os.path.join(HERE, 'generate')
    # Set up some directories:
    make_dirs(exe_dir)
    clean_dir(exe_dir)
    engine.exe_dir = exe_dir
    # Draw random velocities:
    ensemble = {'system': system, 'rgen': None}
    vel_settings = {'aimless': True}
    dek, kin_new = engine.modify_velocities(ensemble, vel_settings)
    # This system did not have a kinetic energy assigned.
    assert dek == float('inf')
    assert math.isclose(kin_new, 1.4970703)
    # The order parameter is the position and velocity of particle 1:
    assert np.allclose(system.order, [1.68543117318838, -0.234327230058829])
    print_to_screen('\n-> First draw worked as expected.', level='success')
    # Now, backup the input/output files so we can inspect
    # them later.
    prefix = 'prev_0_'
    files = backup_files(exe_dir, prefix=prefix)
    # Move the position file:
    filename = os.path.basename(system.particles.get_pos()[0])
    system.particles.set_pos((files[filename], 0))
    # Draw random velocities again:
    ensemble = {'system': system, 'rgen': None}
    vel_settings = {'aimless': True}
    dek, kin_new = engine.modify_velocities(ensemble, vel_settings)
    assert math.isclose(dek, 0.0)
    assert math.isclose(kin_new, 1.4970703)
    # Same seed, velocity should still be the same:
    assert np.allclose(system.order, [1.68543117318838, -0.234327230058829])
    print_to_screen('\n-> Second draw worked as expected.', level='success')
    # Bakcup files again:
    prefix = 'prev_1_'
    files = backup_files(exe_dir, prefix=prefix)
    filename = os.path.basename(system.particles.get_pos()[0])
    system.particles.set_pos((files[filename], 0))
    # Draw random velocities again, but with a different seed:
    rgen = create_random_generator({'seed': 0})
    ensemble = {'system': system, 'rgen': rgen}
    vel_settings = {'aimless': True}
    dek, kin_new = engine.modify_velocities(ensemble, vel_settings)
    assert math.isclose(dek, 0.0)
    assert math.isclose(kin_new, 1.4970703)
    # Different seed, velocity should have changed
    assert np.allclose(system.order, [1.68543117318838, 0.277207562198471])
    print_to_screen('\n-> Third draw worked as expected.', level='success')
    print_to_screen('\nAll draws were successful!', level='success')


def main():
    """Run the comparisons."""
    modify_velocities()


if __name__ == '__main__':
    colorama.init(autoreset=True)
    main()



# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""
Here we test the basic functionality of the CP2K engine.

1) We generate random velocities.

2) We integrate forward in time.

3) We reverse the velocities.

4) We integrate backward in time from 3.
"""
import os
import sys
import shutil
import time
import itertools
import colorama
import numpy as np
from matplotlib import pyplot as plt
from pyretis.core import System, create_box, ParticlesExt, Path
from pyretis.orderparameter import PositionVelocity
from pyretis.inout.common import make_dirs
from pyretis.inout import print_to_screen
from pyretis.inout.settings import parse_settings_file
from pyretis.inout.formats.xyz import read_xyz_file, write_xyz_trajectory
from pyretis.engines.cp2k import (
    CP2KEngine,
    write_for_step_vel,
    convert_snapshot
)


plt.style.use('seaborn-v0_8-deep')


def clean_dir(dirname):
    """Remove ALL files in the given directory."""
    for files in os.listdir(dirname):
        filename = os.path.join(dirname, files)
        if os.path.isfile(filename):
            os.remove(filename)


def run_in_steps(engine, system, order_parameter, interfaces,
                 steps=25, exe_dir='forward-step', reverse=False):
    """Run the engine forward in time, in steps.

    Parameters
    ----------
    engine : object like :py:class:`.ExternalMDEngine
        Engine to use for propagation.
    system : object like :py:class:`.System`
        The system we are propagation.
    order_parameter : object like :py:class:`.OrderParameter`
        An order parameter to calculate.
    interfaces : list of floats
        Interfaces to consider, here typically just set to
        ``[-float('inf'), float('inf'), float('inf')]``.
    steps : integer
        The number of steps we will do.
    exe_dir : string
        The folder to use for the execution.
    reverse : boolean
        Selects the time direction.

    """
    print_to_screen(f'\nRunning {steps} steps in "{exe_dir}"',
                    level='message')
    print_to_screen(f'(Reverse = {reverse})')
    make_dirs(exe_dir)
    folder = os.path.abspath(exe_dir)
    clean_dir(folder)
    engine.exe_dir = folder
    path = Path(None, maxlen=steps)
    start = time.perf_counter()
    engine.propagate(path,
                     {'system': system, 'order_function': order_parameter,
                      'interfaces': interfaces},
                     reverse=reverse)
    end = time.perf_counter()
    print_to_screen('Propagation done!')
    print_to_screen(f'Time spent: {end - start}', level='info')
    return path


def run_plain_cp2k(engine, system, order_parameter, input_conf,
                   steps=25, exe_dir='forward-plain', reverse=False):
    """Run a plain CP2K simulation.

    Parameters
    ----------
    engine : object like :py:class:`.CP2KEngine`
        This is used to obtain the commands for executing CP2K.
    system : object like :py:class:`.System`
        The system we are propagation.
    order_parameter : object like :py:class:`.OrderParameter`
        An order parameter to calculate.
    input_conf : string
        The input configuration to use for the CP2K run.
    steps : integer
        The number of steps to run.
    exe_dir : string
        The path to where we should execute CP2K.
    reverse : boolean
        Selects the time direction.

    """
    print_to_screen(
        f'\nRunning {steps} plain CP2K steps in "{exe_dir}"',
        level='message'
    )
    print_to_screen(f'(Reverse = {reverse})')
    make_dirs(exe_dir)
    folder = os.path.abspath(exe_dir)
    clean_dir(folder)
    engine.exe_dir = exe_dir
    shutil.copy(input_conf, folder)
    _, _, vel, _ = engine._read_configuration(input_conf)
    input_file = os.path.join(exe_dir, 'md.inp')
    write_for_step_vel(
        engine.input_files['template'],
        input_file,
        engine.timestep,
        (steps - 1) * engine.subcycles,
        os.path.basename(input_conf),
        vel,
        name='md',
        print_freq=1
    )
    engine.add_input_files(exe_dir)
    print_to_screen('Running CP2K...')
    start = time.perf_counter()
    engine.run_cp2k('md.inp', 'md')
    end = time.perf_counter()
    print_to_screen('Done!')
    print_to_screen(f'Time spent: {end - start}', level='info')
    energy_file = os.path.join(exe_dir, 'md-1.ener')
    energy = np.loadtxt(energy_file)
    pos_file = os.path.join(exe_dir, 'md-pos-1.xyz')
    vel_file = os.path.join(exe_dir, 'md-vel-1.xyz')
    order = []
    box = np.array([5.0, 5.0, 5.0])
    out_file = os.path.join(exe_dir, 'rev_last.xyz')
    out_file2 = os.path.join(exe_dir, 'rev_last2.xyz')
    for snapshotp, snapshotv in zip(read_xyz_file(pos_file),
                                    read_xyz_file(vel_file)):
        _, xyz, _, names = convert_snapshot(snapshotp)
        _, vel, _, _ = convert_snapshot(snapshotv)  # interpret pos as vel
        system.particles.pos = xyz
        system.particles.vel = vel
        if reverse:
            system.particles.vel *= -1
        system.box.update_size(box)
        order.append(order_parameter.calculate(system))
        write_xyz_trajectory(out_file, xyz, -vel, names, box,
                             append=False)
        write_xyz_trajectory(out_file2, xyz, -vel, names, box,
                             append=False)
    order = np.array(order)
    return energy, order, out_file


def test_genvel(engine, input_file, exe_dir='genvel'):
    """Test generation of velocities.

    Parameters
    ----------
    engine : object like :py:class:`.ExternalMDEngine
        Engine to use for the generation.
    input_file : string
        A file containing the input configuration for CP2K.
    exe_dir : string, optional
        The folder where we will be executing CP2K.

    Returns
    -------
    out_conf : string
        The path to the output configuration with randomized
        velocities.

    """
    print_to_screen(f'\nRunning CP2K genvel step in "{exe_dir}"',
                    level='message')
    make_dirs(exe_dir)
    folder = os.path.abspath(exe_dir)
    clean_dir(folder)
    engine.exe_dir = folder
    start = time.perf_counter()
    out_conf, _ = engine._prepare_shooting_point(input_file)
    end = time.perf_counter()
    print_to_screen('Generation of velocity done!')
    print_to_screen(f'Time spent: {end - start}', level='info')
    return out_conf


def main(plot=False):
    """Run the tests."""
    settings = parse_settings_file('engine.rst')
    steps = settings['simulation']['steps']
    engine_settings = settings['engine']
    engine = CP2KEngine(
        engine_settings['cp2k'],
        engine_settings['input_path'],
        engine_settings['timestep'],
        engine_settings['subcycles'],
        extra_files=engine_settings.get('extra_files', [])
    )
    print_to_screen(f'Testing engine: {engine}', level='info')
    print_to_screen(f'Time step: {engine.timestep}')
    print_to_screen(f'Subcycles: {engine.subcycles}')
    system = System(units='gromacs',
                    box=create_box(cell=[100, 100, 100]),
                    temperature=500)
    system.particles = ParticlesExt(dim=3)
    initial_conf = engine.input_files['conf']
    interfaces = [-float('inf'), float('inf'), float('inf')]
    order_parameter = PositionVelocity(0, dim='x', periodic=True)

    # First generate some random velocities:
    initial_conf = test_genvel(engine, initial_conf, exe_dir='genvel')
    system.particles.set_pos((initial_conf, None))
    system.particles.set_vel(False)

    pathf = run_in_steps(engine, system, order_parameter, interfaces,
                         steps=steps, exe_dir='forward', reverse=False)

    # set state to last point in trajectory:
    phase_point = pathf.phasepoints[-1]
    system = phase_point.copy()

    pathb = run_in_steps(engine, system, order_parameter, interfaces,
                         steps=steps, exe_dir='backward', reverse=True)

    plainf = run_plain_cp2k(engine, system, order_parameter, initial_conf,
                            steps=steps, exe_dir='forward-plain',
                            reverse=False)

    plainb = run_plain_cp2k(engine, system, order_parameter, plainf[-1],
                            steps=steps, exe_dir='backward-plain',
                            reverse=True)
    obtain_mses(pathf, pathb, plainf, plainb, engine.subcycles)
    if plot:
        steps = np.array([i * engine.subcycles for i in range(steps)])
        plot_path_comparison(steps, pathf, pathb, plainf, plainb,
                             engine.subcycles)


def mse_combinations(text, var, tol=None):
    """Calculate mse for several combinations."""
    for comb in itertools.combinations(var, 2):
        mse = ((comb[0][0] - comb[1][0])**2).mean(axis=0)
        level = 'info'
        tol_ok = True
        if tol:
            try:
                tol_ok = all([abs(i) < tol for i in mse])
            except TypeError:
                tol_ok = abs(mse) < tol
            if not tol_ok:
                level = 'error'
        print_to_screen(
            f'MSE {text}: {comb[0][1]} vs {comb[1][1]} = {mse}',
            level=level
        )
        if not tol_ok:
            sys.exit(1)


def obtain_mses(pathf, pathb, plainf, plainb, subcycles):
    """Calculate mean square errors."""
    orderf = [i.order for i in pathf.phasepoints]
    orderb = [i.order for i in reversed(pathb.phasepoints)]
    mses = [(np.array(orderf), 'step-forward'),
            (np.array(orderb), 'step-back'),
            (plainf[1][::subcycles, :], 'plain-forward'),
            (plainb[1][::subcycles, :][::-1], 'plain-back')]
    mse_combinations('order parameters', mses, tol=1e-7)

    ekinf = [i.particles.ekin for i in pathf.phasepoints]
    ekinb = [i.particles.ekin for i in reversed(pathb.phasepoints)]
    mses = [(np.array(ekinf), 'step-forward'),
            (np.array(ekinb), 'step-back'),
            (plainf[0][::subcycles, 2], 'plain-forward'),
            (plainb[0][::subcycles, 2][::-1], 'plain-back')]
    mse_combinations('kinetic energy', mses, tol=1e-7)

    vpotf = [i.particles.vpot for i in pathf.phasepoints]
    vpotb = [i.particles.vpot for i in reversed(pathb.phasepoints)]
    mses = [(np.array(vpotf), 'step-forward'),
            (np.array(vpotb), 'step-back'),
            (plainf[0][::subcycles, 4], 'plain-forward'),
            (plainb[0][::subcycles, 4][::-1], 'plain-back')]
    mse_combinations('potential energy', mses, tol=1e-4)


def plot_path_comparison(steps, pathf, pathb, plainf, plainb, subcycles):
    """Just plot some properties for the paths."""
    orderf = np.array([i.order for i in pathf.phasepoints])
    orderb = np.array([i.order for i in reversed(pathb.phasepoints)])
    fig1 = plt.figure(figsize=(12, 6))
    ax11 = fig1.add_subplot(121)
    ax12 = fig1.add_subplot(122)
    ax11.plot(steps, orderf[:, 0], lw=2, ls='-', marker='o',
              label='Forward', ms=12, alpha=0.8)
    ax11.plot(steps, orderb[:, 0], lw=2, ls='--', marker='^',
              label='Backward', ms=12, alpha=0.8)
    ax11.plot(plainf[1][:, 0], lw=2, ls='--', marker='^',
              label='Plain forward', alpha=0.8)
    ax11.plot(plainb[1][:, 0][::-1], lw=2, ls='-.', marker='x',
              label='Plain backward', alpha=0.8)
    ax11.legend()
    ax11.set_title('Order param 1')
    ax12.plot(steps, orderf[:, 1], lw=2, ls='-', marker='o', ms=12,
              alpha=0.8)
    ax12.plot(steps, orderb[:, 1], lw=2, ls='--', marker='^', ms=12,
              alpha=0.8)
    ax12.plot(plainf[1][:, 1], lw=2, ls=':', marker='s', alpha=0.8)
    ax12.plot(plainb[1][:, 1][::-1], lw=2, ls='-.', marker='x',
              alpha=0.8)
    ax12.set_title('Order param 2')

    ekinf = np.array([i.particles.ekin for i in pathf.phasepoints])
    ekinb = np.array(
        [i.particles.ekin for i in reversed(pathb.phasepoints)]
    )
    fig2 = plt.figure(figsize=(12, 6))
    ax21 = fig2.add_subplot(121)
    ax22 = fig2.add_subplot(122)
    ax21.plot(steps, ekinf, lw=2, ls='-', marker='o', label='Forward', ms=12,
              alpha=0.8)
    ax21.plot(steps, ekinb, lw=2, ls='--', marker='^', label='Backward',
              ms=9, alpha=0.8)
    ax21.plot(plainf[0][:, 0], plainf[0][:, 2], lw=2, ls=':', marker='s',
              label='Plain forward', alpha=0.8)
    ax21.plot(plainb[0][:, 0], plainb[0][:, 2][::-1], lw=2, ls='-.',
              marker='x', label='Plain backward', alpha=0.8)
    ax21.set_title('Kinetic energy')
    ax21.legend()

    vpotf = np.array([i.particles.vpot for i in pathf.phasepoints])
    vpotb = np.array(
        [i.particles.vpot for i in reversed(pathb.phasepoints)]
    )
    ax22.plot(steps, vpotf, lw=2, ls='-', marker='o', label='Forward',
              ms=12, alpha=0.8)
    ax22.plot(steps, vpotb, lw=2, ls='--', marker='^', label='Backward',
              ms=12, alpha=0.8)
    ax22.plot(plainf[0][:, 0], plainf[0][:, 4], lw=2, ls=':', marker='s',
              label='Plain forward', alpha=0.8)
    ax22.plot(plainb[0][:, 0], plainb[0][:, 4][::-1], lw=2, ls='-.',
              marker='x', label='Plain backward', alpha=0.8)
    ax22.set_title('Potential energy')
    ax22.legend()

    fig3 = plt.figure(figsize=(12, 6))
    ax31 = fig3.add_subplot(221)
    ax32 = fig3.add_subplot(222)
    ax33 = fig3.add_subplot(223)
    ax34 = fig3.add_subplot(224)

    ax31.scatter(orderf[:, 0], orderb[:, 0], marker='o', label='Backward',
                 alpha=0.8)
    ax31.scatter(orderf[:, 0], plainf[1][::subcycles, 0], marker='s',
                 label='Forward-plain', alpha=0.8)
    ax31.scatter(orderf[:, 0], plainb[1][::subcycles, 0][::-1], marker='^',
                 label='Backward-plain', alpha=0.8)
    minx = min(orderf[:, 0])
    maxx = max(orderf[:, 0])
    ax31.plot([minx, maxx], [minx, maxx], ls=':',
              c='#262626', alpha=0.5, lw=2)
    ax31.set_xlabel('Order 1 Forward')
    ax31.set_ylabel('Order 1')
    ax31.legend()

    ax32.scatter(orderf[:, 1], orderb[:, 1], marker='o', label='Backward',
                 alpha=0.8)
    ax32.scatter(orderf[:, 1], plainf[1][::subcycles, 1], marker='s',
                 label='Forward-plain', alpha=0.8)
    ax32.scatter(orderf[:, 1], plainb[1][::subcycles, 1][::-1], marker='^',
                 label='Backward-plain', alpha=0.8)
    ax32.set_xlabel('Order 2 Forward')
    ax32.set_ylabel('Order 2')
    minx = min(orderf[:, 1])
    maxx = max(orderf[:, 1])
    ax32.plot([minx, maxx], [minx, maxx], ls=':',
              c='#262626', alpha=0.5, lw=2)
    ax32.legend()

    ax33.scatter(ekinf, ekinb, marker='o', label='Backward',
                 alpha=0.8)
    ax33.scatter(ekinf, plainf[0][::subcycles, 2], marker='s',
                 label='Forward-plain', alpha=0.8)
    ax33.scatter(ekinf, plainb[0][::subcycles, 2][::-1], marker='^',
                 label='Backward-plain', alpha=0.8)
    ax33.set_xlabel('Ekin Forward')
    ax33.set_ylabel('Ekin')
    minx = min(ekinf)
    maxx = max(ekinf)
    ax33.plot([minx, maxx], [minx, maxx], ls=':',
              c='#262626', alpha=0.5, lw=2)
    ax33.legend()

    ax34.scatter(vpotf, vpotb, marker='o', label='Backward',
                 alpha=0.8)
    ax34.scatter(vpotf, plainf[0][::subcycles, 4], marker='s',
                 label='Forward-plain', alpha=0.8)
    ax34.scatter(vpotf, plainb[0][::subcycles, 4][::-1], marker='^',
                 label='Backward-plain', alpha=0.8)
    ax34.set_xlabel('Vpot Forward')
    ax34.set_ylabel('Vpot')
    minx = min(vpotf)
    maxx = max(vpotf)
    ax34.plot([minx, maxx], [minx, maxx], ls=':',
              c='#262626', alpha=0.5, lw=2)
    ax34.legend()

    fig1.tight_layout()
    fig2.tight_layout()
    fig3.tight_layout()

    plt.show()


if __name__ == '__main__':
    colorama.init(autoreset=True)
    PLOT = len(sys.argv) > 1
    main(plot=PLOT)



# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""This module handles the creation of force fields from simulation settings.

Important methods defined here
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

create_potentials (:py:func:`.create_potentials`)
    Method for creating potentials from a dictionary of settings.
    Note that this method will make use of :py:func:`.create_potential`.

create_force_field (:py:func:`.create_force_field`)
    Method to create a force field from input settings.
"""
import logging
from pyretis.setup.common import create_potential
from pyretis.forcefield import ForceField
logger = logging.getLogger(__name__)  # pylint: disable=invalid-name
logger.addHandler(logging.NullHandler())


__all__ = ['create_force_field', 'create_potentials']


def create_potentials(settings):
    """Create potential functions from given simulations settings.

    This method will basically loop over the given potential settings
    and just run :py:func:`.create_potential` for each setting.

    Parameters
    ----------
    settings : dict
        This dictionary contains the settings for the simulation.

    Returns
    -------
    out[0] : list
        A list of potential functions.
    out[1] : list
        A list of parameters for the potential functions.

    """
    potentials = settings.get('potential', [])
    try:
        ndim = settings['system']['dimensions']
    except KeyError:
        ndim = None
    out_pot, out_par = [], []
    for i, pot_settings in enumerate(potentials):
        potential_function = create_potential(settings, pot_settings)
        if potential_function is None:
            msg = 'The following potential settings were ignored!\n{}'
            msgtxt = msg.format(pot_settings)
            logger.warning(msgtxt)
        pdim = getattr(potential_function, 'dim', None)
        if pdim is not None and ndim is not None:
            if ndim != pdim:
                msg = ('Inconsistent dimensions in potential!'
                       '\nSettings gives: {}D, potential {} is {}D')
                msgtxt = msg.format(ndim, i, pdim)
                logger.error(msgtxt)
                raise ValueError(msgtxt)
        out_pot.append(potential_function)
        out_par.append(pot_settings.get('parameter', None))
    return out_pot, out_par


def create_force_field(settings):
    """Create a force field from input settings.

    This method will create the required potential functions with the
    specified parameters from `settings`.

    Parameters
    ----------
    settings : dict
        This dictionary contains the settings for a single potential.

    Returns
    -------
    out : object like :py:class:`.ForceField`
        This object represents the force field.

    """
    try:
        desc = settings['forcefield']['description']
    except KeyError:
        desc = 'Generic force field'
    potentials, pot_param = create_potentials(settings)
    ffield = ForceField(desc, potential=potentials, params=pot_param)
    msg = ['Created force field:', f'{ffield}']
    msgtxt = '\n'.join(msg)
    logger.info(msgtxt)
    return ffield



# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""Test the order parameter classes from pyretis.orderparameter."""
import logging
import unittest
import numpy as np
from pyretis.orderparameter import order_factory
from pyretis.orderparameter import (
    OrderParameter,
    Position,
    Velocity,
    PositionVelocity,
    Distance,
    Distancevel,
    DistanceVelocity,
    CompositeOrderParameter,
    Angle,
    Dihedral,
    Permeability,
    PermeabilityMinusOffset
)
from pyretis.core import System, create_box, Particles
logging.disable(logging.CRITICAL)


class SimpleOrder(OrderParameter):
    """An order parameter which equals the system temperature."""

    def __init__(self):
        super().__init__(description='Simple order parameter')

    def calculate(self, system):
        return [system.temperature['set']]


class SimpleOrderTemp(OrderParameter):
    """An order parameter which equals the system temperature."""

    def __init__(self):
        super().__init__(description='Simple order parameter', velocity=True)

    def calculate(self, system):
        return [system.temperature['set'] * -1]


class SimpleOrderFaulty:  # pylint: disable=too-few-public-methods
    """An order parameter which is faulty - missing calculate."""

    def calculate_all(self, system):  # pylint: disable=no-self-use
        """Just return the set temperature."""
        return [system.temperature['set']]


class SimpleOrderFaulty2:  # pylint: disable=too-few-public-methods
    """An order parameter which is faulty - missing callable calculate."""

    calculate = 100


class OrderGenericTest(unittest.TestCase):
    """Test that we can create a class and define some parameters."""

    def test_simple_order(self):
        """Test that we can create a very simple order parameter."""
        system = System(temperature=123.0, units='lj', box=None)
        order = SimpleOrder()
        correct = [123.0, 'lj']
        val = order.calculate(system)
        self.assertAlmostEqual(val[0], correct[0])
        self.assertEqual(len(val), 1)

        vals = order.calculate(system)
        self.assertAlmostEqual(vals[0], correct[0])
        self.assertEqual(len(vals), 1)


def create_system(ndim, npart, periodic=False):
    """Create a simple system for testing."""
    if periodic:
        box = create_box(low=[0]*ndim, high=[1]*ndim)
    else:
        box = create_box(periodic=[False]*ndim)
    system = System(box=box)
    system.particles = Particles(system.get_dim())
    for _ in range(npart):
        pos = np.random.random(box.dim)
        vel = np.random.random(box.dim)
        system.add_particle(name='Ar', pos=pos, vel=vel)
    return system, box


class OrderPositionTest(unittest.TestCase):
    """Run the tests for the Position class."""

    def _check_order_parameter(self, orderp, correct, system, idim, ndim):
        """Verify the order parameter."""
        for orderpi, correcti in zip(orderp, correct):
            if idim > ndim - 1:
                with self.assertRaises(IndexError):
                    orderpi.calculate(system)
            else:
                for i, j in zip(orderpi.calculate(system), correcti):
                    self.assertAlmostEqual(i, j)

    @staticmethod
    def _get_order(index, xdim, periodic=False):
        """Return the initiated order parameters."""
        orderp = [
            Position(index, dim=xdim, periodic=periodic),
            Velocity(index, dim=xdim),
            PositionVelocity(index, dim=xdim, periodic=periodic),
        ]
        return orderp

    def _get_correct(self, system, index, idim, periodic=False):
        """Return the correct order parameters."""
        correct = [
            self._correct_order1(system, index, idim,
                                 periodic=periodic),
            self._correct_order2(system, index, idim),
        ]
        correct.append(correct[0] + correct[1])
        return correct

    def test_without_pbc(self):
        """Test the order parameters without periodic boundaries."""
        periodic = False
        for npart, index in zip((1, 10), (0, 2)):
            for ndim in [1, 2, 3]:
                system, _ = create_system(ndim, npart, periodic=periodic)
                for idim, xdim in enumerate(('x', 'y', 'z')):
                    orderp = self._get_order(index, xdim, periodic=periodic)
                    correct = self._get_correct(system, index, idim,
                                                periodic=periodic)
                    self._check_order_parameter(orderp, correct, system,
                                                idim, ndim)

    def test_with_pbc(self):
        """Test the order parameters with periodic boundaries."""
        periodic = True
        for npart, index in zip((1, 10), (0, -1)):
            for ndim in [1, 2, 3]:
                # Just test for some displacements:
                for disp in [0.0, 1.5, -1.5, 100., -100.]:
                    system, _ = create_system(ndim, npart, periodic=periodic)
                    system.particles.pos += (
                        np.ones_like(system.particles.pos) * disp
                    )
                    for idim, xdim in enumerate(('x', 'y', 'z')):
                        orderp = self._get_order(index, xdim,
                                                 periodic=periodic)
                        correct = self._get_correct(system, index, idim,
                                                    periodic=periodic)
                        self._check_order_parameter(orderp, correct, system,
                                                    idim, ndim)

    @staticmethod
    def _correct_order1(system, index, idim, periodic=False):
        """The correct position order parameter."""
        try:
            pos = system.particles.pos[index][idim]
            if periodic:
                box = system.box
                return [box.pbc_coordinate_dim(pos, idim)]
            return [pos]
        except IndexError:
            return [None]

    @staticmethod
    def _correct_order2(system, index, idim):
        """The correct velocity order parameter."""
        try:
            return [system.particles.vel[index][idim]]
        except IndexError:
            return [None]

    def test_init_fail(self):
        """Check that the initiation fails if we supply strange input."""
        with self.assertRaises(ValueError):
            Position(0, dim='a')
        with self.assertRaises(ValueError):
            Velocity(0, dim='pingu')
        with self.assertRaises(ValueError):
            PositionVelocity(123, dim='chonky')


class OrderDistanceTest(unittest.TestCase):
    """Run the tests for the Distance class."""

    def _check_order_parameter(self, orderp, correct, system):
        """Verify the order parameter."""
        for orderpi, correcti in zip(orderp, correct):
            for i, j in zip(orderpi.calculate(system), correcti):
                self.assertAlmostEqual(i, j)

    @staticmethod
    def _get_order(index, periodic=False):
        """Return the initiated order parameters."""
        orderp = [
            Distance(index, periodic=periodic),
            Distancevel(index, periodic=periodic),
            DistanceVelocity(index, periodic=periodic),
        ]
        return orderp

    def _get_correct(self, system, index, periodic=False):
        """Return the correct order parameters."""
        correct = [
            self._correct_order1(system, index, periodic=periodic),
            self._correct_order2(system, index, periodic=periodic),
        ]
        correct.append(correct[0] + correct[1])
        return correct

    def test_two_particles(self):
        """Test the distance order parameter without pbc."""
        # Test for a one-particle system:
        index = (0, 1)
        for ndim in [1, 2, 3]:
            system, _ = create_system(ndim, 2, periodic=False)
            orderp = self._get_order(index, periodic=False)
            correct = self._get_correct(system, index, periodic=False)
            self._check_order_parameter(orderp, correct, system)

    def test_two_particles_pbs(self):
        """Test the distance order parameter with pbc."""
        # Test for a one-particle system:
        index = (0, 1)
        for disp in [0.0, 1.5, -1.5, 100., -100.]:
            for ndim in [1, 2, 3]:
                system, box = create_system(ndim, 2, periodic=True)
                for i in index:
                    system.particles.pos[i] = (
                        np.random.random(box.dim) + np.ones(box.dim) * disp
                    )
                orderp = self._get_order(index, periodic=True)
                correct = self._get_correct(system, index, periodic=True)
                self._check_order_parameter(orderp, correct, system)

    @staticmethod
    def _correct_order1(system, index, periodic=False):
        """The correct position order parameter."""
        try:
            i, j = index
            delta = system.particles.pos[j] - system.particles.pos[i]
            if periodic:
                box = system.box
                delta = box.pbc_dist_coordinate(delta)
            return [np.sqrt(np.dot(delta, delta))]
        except IndexError:
            return [None]

    @staticmethod
    def _correct_order2(system, index, periodic=False):
        """The correct velocity order parameter."""
        try:
            i, j = index
            delta = system.particles.pos[j] - system.particles.pos[i]
            delta_v = system.particles.vel[j] - system.particles.vel[i]
            if periodic:
                box = system.box
                delta = box.pbc_dist_coordinate(delta)
            return [np.dot(delta, delta_v) / np.sqrt(np.dot(delta, delta))]
        except IndexError:
            return [None]

    def test_init_fail(self):
        """Check that the initiation fails if we supply strange input."""
        inputs = [0, [0], (0,), (0, 1, 2)]
        errors = [TypeError, ValueError, ValueError, ValueError]
        klasses = (
            Distance,
            Distancevel,
            DistanceVelocity
        )
        for cls in klasses:
            for i, j in zip(inputs, errors):
                with self.assertRaises(j):
                    cls(i)


def water_molecule(box):
    """Return a simple system with a single water molecule."""
    system = System(box=box)
    system.particles = Particles(system.get_dim())
    system.add_particle(name='O', pos=np.array([0.230, 0.628, 0.113]))
    system.add_particle(name='H', pos=np.array([0.137, 0.626, 0.150]))
    system.add_particle(name='H', pos=np.array([0.231, 0.589, 0.021]))
    return system


def triangle():
    """Return a 2D system with particles in a tright riangle."""
    box = create_box(periodic=[False, False])
    system = System(box=box)
    system.particles = Particles(system.get_dim())
    system.add_particle(name='X', pos=np.array([0.0, 0.0]))
    system.add_particle(name='X', pos=np.array([1.0, 0.0]))
    system.add_particle(name='X', pos=np.array([0.0, 1.0]))
    angles = [
        ((1, 0, 2), 90.),
        ((0, 1, 2), 45.),
        ((0, 2, 1), 45.),
    ]
    return system, angles


class OrderAngleTest(unittest.TestCase):
    """Run the tests for the Angle class."""

    def test_without_pbc(self):
        """Test the angle order parameter without pbc."""
        orderp = Angle((1, 0, 2), periodic=False)
        box = create_box(periodic=[False]*3)
        # Test angle for the SPC water geometry.
        system = water_molecule(box)
        angle = orderp.calculate(system)[0]
        self.assertAlmostEqual(np.degrees(angle), 109.984398, places=3)

    def test_witht_pbc(self):
        """Test the angle order parameter with pbc."""
        orderp = Angle((1, 0, 2), periodic=True)
        # Test angle for the SPC water geometry.
        box = create_box(periodic=[True, True, True], cell=[1., 1., 1.])
        system = water_molecule(box)
        angle = orderp.calculate(system)[0]
        self.assertAlmostEqual(np.degrees(angle), 109.984398, places=3)

    def test_triangle(self):
        """Test the angle order parameter for a 2D case."""
        system, angles = triangle()
        for idx, correct in angles:
            orderp = Angle(idx, periodic=False)
            angle = orderp.calculate(system)[0]
            self.assertAlmostEqual(np.degrees(angle), correct)

    def test_initiate_fail(self):
        """Test that we fail if we give incorrect number of indices."""
        with self.assertRaises(TypeError):
            Angle(0, periodic=False)
        with self.assertRaises(ValueError):
            Angle((0,), periodic=False)
        with self.assertRaises(ValueError):
            Angle((0, 1), periodic=False)
        with self.assertRaises(ValueError):
            Angle((0, 1, 2, 3), periodic=False)

    def test_special_cases(self):
        """Test the angle order parameter for some special cases:

        1. The angle between (1, 0, 0) and (0, 1, 0) -> pi/2.
        2. The angle between (1, 0, 0) and (1, 0, 0) -> 0.
        3. The angle between (1, 0, 0) and (-1, 0, 0) -> -pi.

        """
        orderp = Angle((0, 1, 2), periodic=False)
        test_cases = [
            {'angle': 0.5 * np.pi, 'pos': np.array([[1.0, 0.0, 0.0],
                                                    [0.0, 0.0, 0.0],
                                                    [0.0, 1.0, 0.0]])},
            {'angle': 0.0, 'pos': np.array([[1.0, 0.0, 0.0],
                                            [0.0, 0.0, 0.0],
                                            [1.0, 0.0, 0.0]])},
            {'angle': np.pi, 'pos': np.array([[1.0, 0.0, 0.0],
                                              [0.0, 0.0, 0.0],
                                              [-1.0, 0.0, 0.0]])},
        ]
        system, _ = create_system(3, 3, periodic=False)
        for case in test_cases:
            system.particles.pos = case['pos']
            angle = orderp.calculate(system)[0]
            self.assertAlmostEqual(angle, case['angle'])


class OrderDihedralTest(unittest.TestCase):
    """Run the tests for the Dihedral class."""

    test_cases = [
        {'angle': 180.0, 'pos': np.array([[0.0, 1.0, 0.0],
                                          [0.0, 0.0, 0.0],
                                          [1.0, 0.0, 0.0],
                                          [1.0, -1.0, 0.0]])},
        {'angle': 0.0, 'pos': np.array([[0.0, 1.0, 0.0],
                                        [0.0, 0.0, 0.0],
                                        [1.0, 0.0, 0.0],
                                        [1.0, 1.0, 0.0]])},
        {'angle': -90.0, 'pos': np.array([[0.0, 0.0, 1.0],
                                          [0.0, 0.0, 0.0],
                                          [1.0, 0.0, 0.0],
                                          [1.0, 1.0, 0.0]])},
        {'angle': 90.0, 'pos': np.array([[0.0, 0.0, -1.0],
                                         [0.0, 0.0, 0.0],
                                         [1.0, 0.0, 0.0],
                                         [1.0, 1.0, 0.0]])},
        {'angle': -60.0127, 'pos': np.array([[0.354, -2.210, -7.248],
                                             [-0.290, -2.221, -6.483],
                                             [0.472, -2.265, -5.191],
                                             [1.036, -3.090, -5.164]])},
        {'angle': 60.0319, 'pos': np.array([[0.354, -2.210, -7.248],
                                            [-0.290, -2.221, -6.483],
                                            [0.472, -2.265, -5.191],
                                            [1.058, -1.458, -5.122]])},
        {'angle': 0.0, 'pos': np.array([[1.499, -0.043, 0.000],
                                        [2.055, 1.361, 0.000],
                                        [3.481, 1.470, 0.000],
                                        [3.898, 0.528, 0.000]])},
        {'angle': -59.365971, 'pos': np.array([[0.039, -0.028, 0.000],
                                               [1.499, -0.043, 0.000],
                                               [1.956, -0.866, -1.217],
                                               [1.571, -1.903, -1.181]])},
        {'angle': 60.833130, 'pos': np.array([[0.039, -0.028, 0.000],
                                              [1.499, -0.043, 0.000],
                                              [1.956, -0.866, -1.217],
                                              [1.610, -0.425, -2.172]])},
        {'angle': -62.290916, 'pos': np.array([[-0.543, -0.938, 0.000],
                                               [0.039, -0.028, 0.000],
                                               [1.499, -0.043, 0.000],
                                               [1.847, -0.534, 0.928]])},
    ]

    def test_without_pbc(self):
        """Test the angle order parameter without pbc."""
        orderp = Dihedral((3, 2, 1, 0), periodic=False)
        system, _ = create_system(3, 4, periodic=False)
        # Test some pre-defined cases:
        for case in self.test_cases:
            system.particles.pos = case['pos']
            angle = orderp.calculate(system)[0]
            angle_deg = np.degrees(angle)  # pylint: disable=no-member
            self.assertAlmostEqual(angle_deg, case['angle'], places=4)

    def test_with_pbc(self):
        """Test the angle order parameter with pbc."""
        orderp = Dihedral((3, 2, 1, 0), periodic=True)
        system, _ = create_system(3, 4, periodic=True)
        # Define a new box for this test:
        box = create_box(periodic=[True, True, True], cell=[8., 8., 8.])
        system.box = box
        # Test same cases, just displaced.
        for case in self.test_cases:
            displace = np.ones_like(case['pos']) * 9
            system.particles.pos = case['pos'] + displace
            angle = orderp.calculate(system)[0]
            angle_deg = np.degrees(angle)  # pylint: disable=no-member
            self.assertAlmostEqual(angle_deg, case['angle'], places=4)

    def test_order(self):
        """Test if we get the same angle if we reverse indices."""
        order1 = Dihedral((0, 1, 2, 3), periodic=False)
        order2 = Dihedral((3, 2, 1, 0), periodic=False)
        system, _ = create_system(3, 4, periodic=False)
        for _ in range(3):
            system.particles.pos = np.random.rand(4, 3)
            angle1 = order1.calculate(system)[0]
            angle2 = order2.calculate(system)[0]
            self.assertAlmostEqual(angle1, angle2)

    def test_initiate_fail(self):
        """Test that we fail if we give incorrect number of indices."""
        with self.assertRaises(TypeError):
            Dihedral(0, periodic=False)
        with self.assertRaises(ValueError):
            Dihedral((0,), periodic=False)
        with self.assertRaises(ValueError):
            Dihedral((0, 1), periodic=False)
        with self.assertRaises(ValueError):
            Dihedral((0, 1, 2), periodic=False)
        with self.assertRaises(ValueError):
            Dihedral((0, 1, 2, 'tre'), periodic=False)


class OrderFactoryTest(unittest.TestCase):
    """Test the order factory."""

    def test_factory(self):
        """Test that we can create order parameters with the factory."""
        test_cases = [
            {
                'setting': {'class': 'orderparameter'},
                'class': OrderParameter,
            },
            {
                'setting': {'class': 'OrderPARAMetEr'},
                'class': OrderParameter,
            },
            {
                'setting': {'class': 'position', 'index': 0},
                'class': Position,
            },
            {
                'setting': {'class': 'velocity', 'index': 0},
                'class': Velocity,
            },
            {
                'setting': {'class': 'distance', 'index': (0, 1)},
                'class': Distance,
            },
            {
                'setting': {'class': 'distancevel', 'index': (0, 1)},
                'class': Distancevel,
            },
            {
                'setting': {'class': 'PositionVelocity', 'index': 0},
                'class': PositionVelocity,
            },
            {
                'setting': {'class': 'distancevelocity', 'index': (0, 1)},
                'class': DistanceVelocity,
            },
            {
                'setting': {'class': 'angle', 'index': (0, 1, 2)},
                'class': Angle,
            },
            {
                'setting': {'class': 'dihedral', 'index': (0, 1, 2, 3)},
                'class': Dihedral,
            },
        ]
        for case in test_cases:
            orderp = order_factory(case['setting'])
            self.assertIsInstance(orderp, case['class'])


class CollectionTest(unittest.TestCase):
    """Test that we can create collections of order parameters."""

    def test_init(self):
        """Test creation of an object."""
        orderp = CompositeOrderParameter()
        cv1 = Distance((0, 1), periodic=False)
        orderp.add_orderparameter(cv1)
        cv2 = Distance((1, 2), periodic=False)
        orderp.add_orderparameter(cv2)
        cv3 = Distance((0, 2), periodic=False)
        orderp.add_orderparameter(cv3)
        orderp2 = CompositeOrderParameter(order_parameters=[cv1, cv2, cv3])
        for i, j in zip(orderp.order_parameters, orderp2.order_parameters):
            self.assertTrue(i is j)
        system, _ = create_system(3, 3, periodic=False)
        system.particles.pos = np.array([[0.0, 0.0, 0.0],
                                         [0.0, 0.0, 0.1],
                                         [0.0, 0.0, 0.3]])
        order = orderp.calculate(system)
        correct = [0.1, 0.2, 0.3]
        self.assertEqual(len(order), len(correct))
        for i, j in zip(order, correct):
            self.assertAlmostEqual(i, j)

    def test_faulty_input(self):
        """Test if we supply faulty input."""
        orderp = CompositeOrderParameter()

        cv1 = SimpleOrder()
        orderp.add_orderparameter(cv1)
        self.assertTrue(cv1 is orderp.order_parameters[0])
        # Try to add some faulty order parameters.
        cv2 = SimpleOrderFaulty()
        with self.assertRaises(ValueError):
            orderp.add_orderparameter(cv2)
        cv3 = SimpleOrderFaulty2()
        with self.assertRaises(ValueError):
            orderp.add_orderparameter(cv3)

    def test_velocity_dependence(self):
        """Test that the combined is marked as velocity dependent."""
        orderp = CompositeOrderParameter()

        cv1 = SimpleOrder()
        orderp.add_orderparameter(cv1)
        self.assertFalse(orderp.velocity_dependent)

        cv2 = SimpleOrder()
        orderp.add_orderparameter(cv2)
        self.assertFalse(orderp.velocity_dependent)

        cv3 = SimpleOrderTemp()
        orderp.add_orderparameter(cv3)
        self.assertTrue(orderp.velocity_dependent)

        cv4 = SimpleOrder()
        orderp.add_orderparameter(cv4)
        self.assertTrue(orderp.velocity_dependent)


class TestPermeability(unittest.TestCase):
    def setUp(self):
        self.system, self.box = create_system(2, 2, periodic=True)
        self.op = Permeability(index=0, dim='x', relative=False)
        self.x = self.system.particles.pos[0, 0]

    def test_non_relative_offset(self):
        with self.assertRaisesRegex(ValueError, "offset"):
            Permeability(index=0, dim='x', offset=-1.01)
        # Test that we do pss if not relative
        _ = Permeability(index=0, dim='x', offset=-1.01, relative=False)

    def test_non_relative_mirror(self):
        with self.assertRaisesRegex(ValueError, "mirror_pos"):
            Permeability(index=0, dim='x', mirror_pos=1.01)
        # Test that we do pss if not relative
        _ = Permeability(index=0, dim='x', mirror_pos=1.01, relative=False)

    def test_x_calculation(self):
        # Test that this is just the position
        self.assertEqual(self.op.calculate(self.system), [self.x, 0, 1])

    def test_x_wrap(self):
        # set x to a value, that +ofsett will be wrapped
        op1 = Permeability(index=0, dim='x')
        op2 = Permeability(index=0, dim='x', offset=0.5)
        x = 0.8
        self.system.particles.pos[0, 0] = x
        self.assertEqual(op1.calculate(self.system)[0], x)
        self.assertAlmostEqual(op2.calculate(self.system)[0], 0.3)

    def test_box_min_under_0(self):
        # set new box
        self.system.box.length[0] = 2
        self.system.box.low[0] = -1
        self.assertAlmostEqual(self.op.calculate(self.system)[0], self.x)
        op2 = Permeability(index=0, dim='x', relative=True)
        # x should be 0.5 + x/2 (adding a box lenght to the left)
        self.assertAlmostEqual(op2.calculate(self.system)[0], 0.5+self.x/2)

    def test_broken_box(self):
        # Break box
        self.system.box = None
        # Make an OP that would normally break the box

        op2 = Permeability(index=0, dim='x', relative=False, offset=-12)
        self.op.periodic = False
        op2.periodic = False

        self.assertAlmostEqual(self.op.calculate(self.system)[0], self.x)
        self.assertAlmostEqual(op2.calculate(self.system)[0], self.x-12)

    def test_mirrored_function(self):
        self.assertFalse(self.op._mirror)
        self.assertAlmostEqual(self.op.calculate(self.system)[0], self.x)
        self.assertAlmostEqual(self.op.calculate(self.system)[2], 1)
        self.op.mirror()
        self.assertTrue(self.op._mirror)
        self.assertAlmostEqual(self.op.calculate(self.system)[0], 1-self.x)
        self.assertAlmostEqual(self.op.calculate(self.system)[2], -1)

    def test_mirror_composite(self):
        orderp = CompositeOrderParameter()
        self.assertFalse(self.op._mirror)
        orderp.add_orderparameter(self.op)
        op2 = Permeability(index=0, dim='x', relative=True)
        op2.mirror()
        self.assertTrue(op2._mirror)
        orderp.add_orderparameter(op2)
        orderp.mirror()
        self.assertTrue(self.op._mirror)
        self.assertFalse(op2._mirror)

    def test_mirror_composite_warning(self):
        orderp = CompositeOrderParameter()
        op1 = Position(index=0)
        orderp.add_orderparameter(op1)
        self.assertFalse(self.op._mirror)
        orderp.add_orderparameter(self.op)
        op2 = Permeability(index=0, dim='x', relative=True)
        op2.mirror()
        self.assertTrue(op2._mirror)
        orderp.add_orderparameter(op2)
        ln = 'pyretis.orderparameter.orderparameter'
        logging.disable(logging.INFO)

        with self.assertLogs(ln, level='INFO') as cm:
            orderp.mirror()
        logging.disable(logging.CRITICAL)
        self.assertIn("Attempting a mirror move, but", cm.output[0])
        self.assertTrue(self.op._mirror)
        self.assertFalse(op2._mirror)

    def test_index_get_set_composite(self):
        orderp = CompositeOrderParameter()
        op1 = Position(index=1)
        orderp.add_orderparameter(op1)
        orderp.add_orderparameter(self.op)
        self.assertEqual(orderp.index, 1)
        orderp.index = 12
        self.assertEqual(op1.index, 12)
        self.assertEqual(self.op.index, 0)

    def test_permeabilityminusoffset(self):
        # Tak e offset that is bigger than the box (normally be wrapped)
        offset = 10
        opmin = PermeabilityMinusOffset(index=0, dim='x', relative=False)
        # Check that with an offset of 0, this is equal
        self.assertEqual(self.op.calculate(self.system),
                         opmin.calculate(self.system))
        self.op.offset = offset
        opmin.offset = offset
        self.assertEqual(self.op.calculate(self.system)[0],
                         opmin.calculate(self.system)[0] + offset)

    def test_restart_cycle(self):
        # See if all the info is there
        info = self.op.restart_info()
        self.assertEqual(info, {"index": 0, "mirror": False})
        # See if mirror is given properly
        self.op.mirror()
        info = self.op.restart_info()
        self.assertEqual(info, {"index": 0, "mirror": True})
        # See if index is given properly
        self.op.index = "bla"
        info = self.op.restart_info()
        self.assertEqual(info, {"index": 'bla', "mirror": True})
        # See if the restart is done properly
        info = {"index": "blub", "mirror": "Fish"}
        self.op.load_restart_info(info)
        self.assertEqual(self.op.index, "blub")
        self.assertEqual(self.op._mirror, "Fish")

    def test_restart_cycle_composite(self):
        orderp = CompositeOrderParameter()
        op1 = Position(index=1)
        orderp.add_orderparameter(op1)
        orderp.add_orderparameter(self.op)
        info = orderp.restart_info()
        ref_info = [None, {'index': 0, 'mirror': False}]
        self.assertEqual(info, ref_info)
        # See if we can set properly as well
        info = [None, {'index': 42, 'mirror': 'towel'}]
        orderp.load_restart_info(info)
        self.assertEqual(self.op.index, 42)
        self.assertEqual(self.op._mirror, 'towel')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""Test the FORTRAN implementation of the Lennard Jones potential.

This test is comparing the three versions of the Lennard Jones
potential:

1) The pure python implementation

2) The numpy python implementation

3) The FORTRAN implementation.
"""
# pylint: disable=invalid-name
import unittest
import numpy as np
from pyretis.core import System, create_box, Particles
from pyretis.core.units import create_conversion_factors
from pyretis.forcefield import ForceField
from pyretis.forcefield.potentials import PairLennardJonesCut
from pyretis.forcefield.potentials import PairLennardJonesCutnp
from pyretis.tools import generate_lattice
from ljpotentialfp import PairLennardJonesCutFp


def set_up_initial_state():
    """Create particles for the test."""
    create_conversion_factors('lj')
    lattice, size = generate_lattice('fcc', [5, 5, 5], density=0.9)
    npart = len(lattice)
    lattice += np.random.randn(npart, 3) * 0.05
    size = np.array(size)
    box = create_box(low=size[:, 0], high=size[:, 1],
                     periodic=[True, True, True])
    system = System(temperature=1.0, units='lj', box=box)
    system.particles = Particles(dim=3)
    for pos in lattice:
        system.add_particle(name='Ar', pos=pos, mass=1.0, ptype=0)
    msg = 'Created lattice with {} atoms.'
    print(msg.format(system.particles.npart))
    return system


def run_calculations(system, parameters):
    """Evaluate the LJ potential."""
    # Calculate with FORTRAN:
    potential_ext = PairLennardJonesCutFp(dim=3, shift=True,
                                          mixing='geometric')
    forcefield_ext = ForceField('Python with external FORTRAN force field',
                                potential=[potential_ext],
                                params=[parameters])
    system.forcefield = forcefield_ext
    print(f'Evaluating with: {forcefield_ext.print_potentials()}')
    vpot_ext, forces_ext, virial_ext = system.potential_and_force()
    vpot_ext /= float(system.particles.npart)
    # Calculate with pure python implementation:
    potential = PairLennardJonesCut(dim=3, shift=True)
    forcefield = ForceField('Pure Python force field',
                            potential=[potential],
                            params=[parameters])
    system.forcefield = forcefield
    print(f'Evaluating with: {forcefield.print_potentials()}')
    vpot, forces, virial = system.potential_and_force()
    vpot /= float(system.particles.npart)
    # Calculate with numpy python implementation:
    potentialnp = PairLennardJonesCutnp(dim=3, shift=True)
    forcefieldnp = ForceField('Python force field with numpy',
                              potential=[potentialnp],
                              params=[parameters])
    system.forcefield = forcefieldnp
    print(f'Evaluating with: {forcefieldnp.print_potentials()}')
    vpotnp, forcesnp, virialnp = system.potential_and_force()
    vpotnp /= float(system.particles.npart)
    return ((vpot, forces, virial),
            (vpotnp, forcesnp, virialnp),
            (vpot_ext, forces_ext, virial_ext))


class LennardJonesTest(unittest.TestCase):
    """Run the tests for the FORTRAN potential class."""

    def test_lj(self):
        """Test one-component system."""
        print('\nTesting for a one-component system')
        system = set_up_initial_state()
        param = {0: {'sigma': 1.0, 'epsilon': 1.0, 'rcut': 2.5}}
        maxcut = 0.5 * min(system.box.length)
        for key in param:
            if 'rcut' in param[key]:
                self.assertGreaterEqual(maxcut, param[key]['rcut'])
        result = run_calculations(system, param)
        keys = ['python', 'python-numpy', 'fortran']
        for i, keyi in enumerate(keys[:-1]):
            for j, key2 in enumerate(keys[i+1:]):
                print(f'\nCompare {keyi} and {key2}')
                force = np.allclose(result[i][1], result[i+j+1][1])
                print(f' -> Forces close: {force}')
                self.assertTrue(force)
                virial = np.allclose(result[i][2], result[i+j+1][2])
                print(f' -> Virial close: {virial}')
                self.assertTrue(virial)
                self.assertAlmostEqual(result[i][0], result[i+j+1][0], 7)
                vdiff = np.abs(result[i][0] - result[i+j+1][0])
                print(f' -> Difference in pot. energy: {vdiff:.15e}')

    def test_lj_mix(self):
        """Test for mixture."""
        print('\nTesting for a two-component mixture')
        system = set_up_initial_state()
        param = {0: {'sigma': 1.0, 'epsilon': 1.0, 'rcut': 2.5},
                 1: {'sigma': 2.0, 'epsilon': 1.2, 'rcut': 3.5}}
        idx = [i for i in range(system.particles.npart)]
        idx2 = np.random.choice(idx, size=int(system.particles.npart * 0.5),
                                replace=False)
        maxcut = 0.5 * min(system.box.length)
        for key in param:
            if 'rcut' in param[key]:
                self.assertGreaterEqual(maxcut, param[key]['rcut'])
        print(f'Mutating {len(idx2)} particles')
        for i in idx2:
            system.particles.ptype[i] = 1
        result = run_calculations(system, param)
        keys = ['python', 'python-numpy', 'fortran']
        for i, keyi in enumerate(keys[:-1]):
            for j, key2 in enumerate(keys[i+1:]):
                print(f'\nCompare {keyi} and {key2}')
                force = np.allclose(result[i][1], result[i+j+1][1])
                print(f' -> Forces close: {force}')
                self.assertTrue(force)
                virial = np.allclose(result[i][2], result[i+j+1][2])
                print(f' -> Virial close: {virial}')
                self.assertTrue(virial)
                self.assertAlmostEqual(result[i][0], result[i+j+1][0], 7)
                vdiff = np.abs(result[i][0] - result[i+j+1][0])
                print(f' -> Difference in pot. energy: {vdiff:.15e}')

    def test_lj_multi_mix(self):
        """Test for multi-mixture."""
        ncomp = np.random.randint(3, 11)
        print(f'\nTesting for a {ncomp}-component mixture')
        system = set_up_initial_state()
        param = {0: {'sigma': 1.0, 'epsilon': 1.0, 'rcut': 2.5}}
        maxcut = 0.5 * min(system.box.length)
        self.assertGreaterEqual(maxcut, param[0]['rcut'])

        idx = np.array([i for i in range(system.particles.npart)],
                       dtype='i4')
        np.random.shuffle(idx)
        for i, idx2 in enumerate(np.array_split(idx, ncomp)):
            system.particles.ptype[idx2] = i
            if i not in param:
                param[i] = {'sigma': np.random.uniform(low=0.5, high=1.5),
                            'epsilon': np.random.uniform(low=0.5, high=1.5),
                            'rcut': np.random.uniform(low=2.0, high=maxcut)}
        natoms = {}
        for i in range(system.particles.npart):
            ptype = system.particles.ptype[i]
            if ptype not in natoms:
                natoms[ptype] = 0
            natoms[ptype] += 1
        for atom in natoms:
            print(f'{natoms[atom]} atoms of type {atom}')
        result = run_calculations(system, param)
        keys = ['python', 'python-numpy', 'fortran']
        for i, keyi in enumerate(keys[:-1]):
            for j, key2 in enumerate(keys[i+1:]):
                print(f'\nCompare {keyi} and {key2}')
                force = np.allclose(result[i][1], result[i+j+1][1])
                print(f' -> Forces close: {force}')
                self.assertTrue(force)
                virial = np.allclose(result[i][2], result[i+j+1][2])
                print(f' -> Virial close: {virial}')
                self.assertTrue(virial)
                self.assertAlmostEqual(result[i][0], result[i+j+1][0], 7)
                vdiff = np.abs(result[i][0] - result[i+j+1][0])
                print(f' -> Difference in pot. energy: {vdiff:.15e}')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2026, PyRETIS Development Team.
# Distributed under the LGPLv2.1+ License. See LICENSE for more info.
"""Dummy order parameter for running tests."""
from pyretis.orderparameter import OrderParameter


class FooOrderParameter(OrderParameter):  # pylint: disable=abstract-method
    """FooOrderParameter(OrderParameter) - Dummy order parameter for tests."""

    def __init__(self, name, desc='Dummy order parameter'):
        super().__init__(description=desc)
        self.name = name


class BarOrderParameter:  # pylint: disable=too-few-public-methods
    """BarOrderParameter - Dummy order parameter for tests."""

    def __init__(self, description='Dummy test order parameter'):
        self.description = description


class BazOrderParameter:  # pylint: disable=too-few-public-methods
    """BazOrderParameter - Dummy order parameter for tests."""

    def __init__(self, description='Dummy test order parameter'):
        self.description = description

    def calculate(self, system):
        """Obtain the order parameter."""
